from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
User = get_user_model()

from rest_framework import permissions, authentication, generics
from rest_framework.response import Response

from .models import Rating, Vote, Comment
from .serializers import *  # noqa

from nodeshot.core.base.mixins import CustomDataMixin
from nodeshot.core.nodes.models import Node
from nodeshot.core.layers.models import Layer


def get_queryset_or_404(queryset, kwargs):
    """
    Checks if object returned by queryset exists
    """
    # ensure exists
    try:
        obj = queryset.get(**kwargs)
    except Exception:
        raise Http404(_('Not found'))
    return obj


class AllNodesParticipationList(generics.ListAPIView):
    """
    Retrieve participation details for all nodes
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    model = Node
    serializer_class = NodeParticipationSerializer
    pagination_serializer_class = PaginationSerializer
    paginate_by_param = 'limit'
    paginate_by = 10

all_nodes_participation = AllNodesParticipationList.as_view()


class AllNodesCommentList(generics.ListAPIView):
    """
    Retrieve comments  for all nodes
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    model = Node
    serializer_class = NodeCommentSerializer
    pagination_serializer_class = PaginationSerializer
    paginate_by_param = 'limit'
    paginate_by = 10

all_nodes_comments = AllNodesCommentList.as_view()


class LayerNodesCommentList(generics.ListAPIView):
    """
    Retrieve comments  for all nodes of a layer
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    model = Node
    serializer_class = NodeCommentSerializer

    def get(self, request, *args, **kwargs):
        """
        Get comments of specified existing layer
        or otherwise return 404
        """
        # ensure layer exists
        layer = get_queryset_or_404(Layer.objects.published(), {'slug': self.kwargs.get('slug', None)})

        # Get queryset of nodes related to layer
        self.queryset = Node.objects.published().filter(layer_id=layer.id)

        return self.list(request, *args, **kwargs)

layer_nodes_comments = LayerNodesCommentList.as_view()


class LayerNodesParticipationList(generics.ListAPIView):
    """
    Retrieve participation details for all nodes of a layer
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    model = Node
    serializer_class = NodeParticipationSerializer

    def get(self, request, *args, **kwargs):
        """
        Get comments of specified existing layer
        or otherwise return 404
        """
        # ensure layer exists
        layer = get_queryset_or_404(Layer.objects.published(), {'slug': self.kwargs.get('slug', None)})
        # Get queryset of nodes related to layer
        self.queryset = Node.objects.published().filter(layer_id=layer.id)
        return self.list(request, *args, **kwargs)

layer_nodes_participation = LayerNodesParticipationList.as_view()


class NodeParticipationDetail(generics.RetrieveAPIView):
    """
    Retrieve participation details for a node
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    model = Node
    serializer_class = NodeParticipationSerializer

node_participation = NodeParticipationDetail.as_view()


class NodeParticipationSettingsDetail(generics.RetrieveAPIView):
    """
    Retrieve participation settings for a node
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    model = Node
    serializer_class = NodeParticipationSettingsSerializer

node_participation_settings = NodeParticipationSettingsDetail.as_view()


class LayerParticipationSettingsDetail(generics.RetrieveAPIView):
    """
    Retrieve participation settings for a layer
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    model = Layer
    serializer_class = LayerParticipationSettingsSerializer

layer_participation_settings = LayerParticipationSettingsDetail.as_view()


class NodeCommentList(CustomDataMixin, generics.ListCreateAPIView):
    """
    Retrieve a **list** of comments for the specified node

    ### POST

    Add a comment to the specified node
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    serializer_class = CommentListSerializer
    serializer_custom_class = CommentAddSerializer

    def get_custom_data(self):
        """ additional request.DATA """
        return {
            'node': self.node.id,
            'user': self.request.user.id
        }

    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure node exists and store it in an instance attribute
            * change queryset to return only comments of current node
        """
        super(NodeCommentList, self).initial(request, *args, **kwargs)
        # ensure node exists
        self.node = get_queryset_or_404(Node.objects.published(), {'slug': self.kwargs.get('slug', None)})
        # return only comments of current node
        self.queryset = Comment.objects.filter(node_id=self.node.id)

node_comments = NodeCommentList.as_view()


class NodeRatingList(CustomDataMixin, generics.CreateAPIView):
    """
    Not allowed

    ### POST

    Add a rating for the specified node
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    serializer_class = RatingListSerializer
    serializer_custom_class = RatingAddSerializer

    def get_custom_data(self):
        """ additional request.DATA """
        return {
            'node': self.node.id,
            'user': self.request.user.id
        }

    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure node exists and store it in an instance attribute
            * change queryset to return only comments of current node
        """
        super(NodeRatingList, self).initial(request, *args, **kwargs)
        # ensure node exists
        self.node = get_queryset_or_404(Node.objects.published(), {'slug': self.kwargs.get('slug', None)})
        # return only comments of current node
        self.queryset = Rating.objects.filter(node_id=self.node.id)

node_ratings = NodeRatingList.as_view()


class NodeVotesList(CustomDataMixin, generics.CreateAPIView):
    """
    Add a vote for the specified node
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    serializer_class = VoteListSerializer
    serializer_custom_class = VoteAddSerializer

    def get_custom_data(self):
        """ additional request.DATA """
        return {
            'node': self.node.id,
            'user': self.request.user.id
        }

    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure node exists and store it in an instance attribute
            * change queryset to return only comments of current node
        """
        super(NodeVotesList, self).initial(request, *args, **kwargs)
        # ensure node exists
        self.node = get_queryset_or_404(Node.objects.published(), {'slug': self.kwargs.get('slug', None)})
        # return only comments of current node
        self.queryset = Vote.objects.filter(node_id=self.node.id)

    def delete(self, *args, **kwargs):
        votes = Vote.objects.filter(node_id=self.node.id, user_id=self.request.user.id)
        if (len(votes) > 0):
            for vote in votes:
                vote.delete()
            message = _('Vote for this node removed')
            status = 200
        else:
            message = _('User has not voted this node yet')
            status = 400
        return Response({'details': message}, status=status)

node_votes = NodeVotesList.as_view()



from rest_framework import pagination, serializers
from rest_framework_gis import serializers as gis_serializers

from nodeshot.core.base.serializers import DynamicRelationshipsMixin

from .models import *


__all__ = [
    'LinkListSerializer',
    'LinkDetailSerializer',
    'LinkListGeoJSONSerializer',
    'LinkDetailGeoJSONSerializer',
    'PaginatedLinkSerializer',
]


class LinkListSerializer(gis_serializers.GeoModelSerializer):
    """ location serializer  """
    layer = serializers.Field('layer_slug')
    quality = serializers.Field(source='quality')
    status = serializers.Field(source='get_status_display')
    type = serializers.Field(source='get_type_display')
    node_a_name = serializers.Field(source='node_a_name')
    node_b_name = serializers.Field(source='node_b_name')

    class Meta:
        model = Link
        fields = [
            'id',
            'layer',
            'node_a_name', 'node_b_name',
            'status', 'type', 'line',
            'metric_type', 'metric_value',
        ]


class LinkListGeoJSONSerializer(LinkListSerializer, gis_serializers.GeoFeatureModelSerializer):
    class Meta:
        model = Link
        geo_field = 'line'
        fields = LinkListSerializer.Meta.fields[:]


class LinkDetailSerializer(DynamicRelationshipsMixin, LinkListSerializer):
    interface_a_mac = serializers.Field(source='interface_a_mac')
    interface_b_mac = serializers.Field(source='interface_b_mac')
    relationships = serializers.SerializerMethodField('get_relationships')

    # this is needed to avoid adding stuff to DynamicRelationshipsMixin
    _relationships = {}

    class Meta:
        model = Link
        fields = [
            'id',
            'layer',
            'node_a_name', 'node_b_name',
            'interface_a_mac', 'interface_b_mac',
            'status', 'type', 'line',
            'quality', 'metric_type', 'metric_value',
            'max_rate', 'min_rate', 'dbm', 'noise',
            'first_seen', 'last_seen',
            'added', 'updated', 'relationships'
        ]

LinkDetailSerializer.add_relationship(
    'node_a',
    view_name='api_node_details',
    lookup_field='node_a_slug'
)

LinkDetailSerializer.add_relationship(
    'node_b',
    view_name='api_node_details',
    lookup_field='node_b_slug'
)


class LinkDetailGeoJSONSerializer(LinkDetailSerializer, gis_serializers.GeoFeatureModelSerializer):
    class Meta:
        model = Link
        geo_field = 'line'
        fields = LinkDetailSerializer.Meta.fields[:]


class PaginatedLinkSerializer(pagination.PaginationSerializer):
    class Meta:
        object_serializer_class = LinkListSerializer



from django.conf import settings


DISCOVERY = getattr(settings, 'NODESHOT_OPEN311_DISCOVERY', {
    'changeset':'2014-02-03 14:18',
    'contact':'email or phone number for assistance',
    'key_service':'URL for api_key requests',
    'endpoints':[
        {
            'specification':'http://wiki.open311.org/GeoReport_v2',
            'url':'Public URL of your endpoint',
            'changeset':'2014-02-03 09:01',
            'type':'production',
            'formats':[
                'application/json'
            ]
        },

    ]
})
METADATA = getattr(settings, 'NODESHOT_OPEN311_METADATA', 'true')
TYPE = getattr(settings, 'NODESHOT_OPEN311_TYPE', 'realtime')
STATUS = getattr(settings, 'NODESHOT_OPEN311_STATUS', {
    'potential': 'open',
    'planned': 'open',
    'active': 'closed',
})



import json
from django.shortcuts import render

from rest_framework.renderers import JSONRenderer

from nodeshot.core.layers.models import Layer
from nodeshot.core.nodes.models import Status
from nodeshot.core.cms.models import MenuItem
from nodeshot.core.layers.serializers import LayerDetailSerializer
from nodeshot.core.nodes.serializers import StatusListSerializer
from nodeshot.core.cms.serializers import MenuSerializer
from nodeshot.community.profiles.serializers import ProfileOwnSerializer

from nodeshot.core.nodes.settings import HSTORE_SCHEMA as NODES_HSTORE_SCHEMA
from . import settings as ui_settings


NODES_HSTORE_SCHEMA = NODES_HSTORE_SCHEMA[:] if NODES_HSTORE_SCHEMA is not None else []
for field in NODES_HSTORE_SCHEMA:
    try:
        field['label'] = field['kwargs']['verbose_name']
    except KeyError:
        field['label'] = field['name'].capitalize().replace('_', ' ')

if 'nodeshot.networking.links' in ui_settings.settings.INSTALLED_APPS:
    LINKS_ENABLED = True
    from nodeshot.networking.links.utils import links_legend
else:
    LINKS_ENABLED = False


def index(request):
    # django-rest-framework serializer context
    serializer_context = {'request': request}
    # load models
    layers = Layer.objects.published()
    status = Status.objects.all()
    menu = MenuItem.objects.published().filter(parent=None).accessible_to(request.user)
    # initialize serializers
    layers = LayerDetailSerializer(layers, many=True, context=serializer_context).data
    status = StatusListSerializer(status, many=True, context=serializer_context).data
    menu = MenuSerializer(menu, many=True, context=serializer_context).data
    # initialize user serializer if authenticated
    if request.user.is_authenticated():
        user = ProfileOwnSerializer(request.user, many=False, context=serializer_context).data
    else:
        user = {}
    # add link legend
    if LINKS_ENABLED:
        status += links_legend
    # initialize django-rest-framework JSON renderer
    json_renderer = JSONRenderer()
    # template context
    context = {
        # preloaded data
        'layers': json_renderer.render(layers),
        'legend': json_renderer.render(status),
        'menu': json_renderer.render(menu),
        'user': json_renderer.render(user),
        # settings
        'SITE_NAME': ui_settings.settings.SITE_NAME,
        'SITE_URL': ui_settings.settings.SITE_URL,
        'MAP_CENTER': ui_settings.MAP_CENTER,
        'MAP_ZOOM': ui_settings.MAP_ZOOM,
        'LEAFLET_OPTIONS': json.dumps(ui_settings.LEAFLET_OPTIONS),
        'DISABLE_CLUSTERING_AT_ZOOM': ui_settings.DISABLE_CLUSTERING_AT_ZOOM,
        'MAX_CLUSTER_RADIUS': ui_settings.MAX_CLUSTER_RADIUS,
        'DATETIME_FORMAT': ui_settings.DATETIME_FORMAT,
        'DATE_FORMAT': ui_settings.DATE_FORMAT,
        'ADDRESS_SEARCH_TRIGGERS': ui_settings.ADDRESS_SEARCH_TRIGGERS,
        'LOGO': ui_settings.LOGO,
        # participation settings
        'VOTING_ENABLED': ui_settings.VOTING_ENABLED,
        'RATING_ENABLED': ui_settings.RATING_ENABLED,
        'COMMENTS_ENABLED': ui_settings.COMMENTS_ENABLED,
        'CONTACTING_ENABLED': ui_settings.CONTACTING_ENABLED,
        # map features (TODO: currently unimplemented)
        'MAP_3D_ENABLED': ui_settings.MAP_3D_ENABLED,
        'MAP_TOOLS_ENABLED': ui_settings.MAP_TOOLS_ENABLED,
        'MAP_PREFERENCES_ENABLED': ui_settings.MAP_PREFERENCES_ENABLED,
        # social auth settings
        'SOCIAL_AUTH_ENABLED': ui_settings.SOCIAL_AUTH_ENABLED,
        'FACEBOOK_ENABLED': ui_settings.FACEBOOK_ENABLED,
        'GOOGLE_ENABLED': ui_settings.GOOGLE_ENABLED,
        'GITHUB_ENABLED': ui_settings.GITHUB_ENABLED,
        # websockets settings
        'WEBSOCKETS': ui_settings.WEBSOCKETS,
        # profiles settings
        'REGISTRATION_OPEN': ui_settings.REGISTRATION_OPEN,
        # additional node fields
        'NODES_HSTORE_SCHEMA': json.dumps(NODES_HSTORE_SCHEMA if NODES_HSTORE_SCHEMA else []),
        # analytics
        'GOOGLE_ANALYTICS_UA': ui_settings.GOOGLE_ANALYTICS_UA,
        'GOOGLE_ANALYTICS_OPTIONS': json.dumps(ui_settings.GOOGLE_ANALYTICS_OPTIONS),
        'PIWIK_ANALYTICS_BASE_URL': ui_settings.PIWIK_ANALYTICS_BASE_URL,
        'PIWIK_ANALYTICS_SITE_ID': ui_settings.PIWIK_ANALYTICS_SITE_ID,
        # miscellaneous
        'ADDITIONAL_GEOJSON_URLS': json.dumps(ui_settings.ADDITIONAL_GEOJSON_URLS),
        'PRIVACY_POLICY_LINK': ui_settings.PRIVACY_POLICY_LINK,
        'TERMS_OF_SERVICE_LINK': ui_settings.TERMS_OF_SERVICE_LINK,
        # networking
        'LINKS_ENABLED': LINKS_ENABLED,
        # metrics
        'METRICS_ENABLED': ui_settings.METRICS_ENABLED
    }
    return render(request, 'index.html', context)



from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.community.participation.views',  # noqa
    url(r'^layers/(?P<slug>[-\w]+)/comments/$', 'layer_nodes_comments', name='api_layer_nodes_comments'),
    url(r'^layers/(?P<slug>[-\w]+)/participation/$', 'layer_nodes_participation', name='api_layer_nodes_participation'),
    url(r'^comments/$', 'all_nodes_comments', name='api_all_nodes_comments'),
    url(r'^participation/$', 'all_nodes_participation', name='api_all_nodes_participation'),
    url(r'^nodes/(?P<slug>[-\w]+)/comments/$', 'node_comments', name='api_node_comments'),
    url(r'^nodes/(?P<slug>[-\w]+)/ratings/$', 'node_ratings', name='api_node_ratings'),
    url(r'^nodes/(?P<slug>[-\w]+)/votes/$', 'node_votes', name='api_node_votes'),
    url(r'^nodes/(?P<slug>[-\w]+)/participation/$', 'node_participation', name='api_node_participation'),
    url(r'^nodes/(?P<slug>[-\w]+)/participation_settings/$', 'node_participation_settings', name='api_node_participation_settings'),
    url(r'^layers/(?P<slug>[-\w]+)/participation_settings/$', 'layer_participation_settings', name='api_layer_participation_settings'),
)



from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies=[
        'django_hstore',
        'nodeshot.core.layers',
        'nodeshot.core.nodes',
    ],
    module='nodeshot.interop.sync'
)


from .layer_external import LayerExternal
from .node_external import NodeExternal


__all__ = ['LayerExternal', 'NodeExternal']


# ------ patch LayerNodesList view to support external layers ------ #

from nodeshot.core.layers.views import LayerNodesList

def get_nodes(self, request, *args, **kwargs):
    try:
        external = self.layer.external
    except LayerExternal.DoesNotExist:
        external = False
    # override view get_nodes method if we have a custom one
    if external and self.layer.is_external and hasattr(external, 'get_nodes'):
        return external.get_nodes(self.__class__.__name__, request.QUERY_PARAMS)
    # otherwise return the standard one
    else:
        return (self.list(request, *args, **kwargs)).data

LayerNodesList.get_nodes = get_nodes



from __future__ import absolute_import

import simplejson as json
from django.contrib.gis.geos import GEOSGeometry
from .base import GenericGisSynchronizer


class GeoJson(GenericGisSynchronizer):
    """ GeoJSON synchronizer """

    def parse(self):
        """ parse geojson and ensure is collection """
        try:
            self.parsed_data = json.loads(self.data)
        except Exception as e:
            raise Exception('Error while converting response from JSON to python. %s' % e)

        if self.parsed_data.get('type', '') != 'FeatureCollection':
            raise Exception('GeoJson synchronizer expects a FeatureCollection object at root level')

        self.parsed_data = self.parsed_data['features']

    def parse_item(self, item):
        result = {
            "name": item['properties'].pop(self.keys['name'], ''),
            "status": item['properties'].pop(self.keys['status'], None),
            "address": item['properties'].pop(self.keys['address'], ''),
            "is_published": item['properties'].pop(self.keys['is_published'], True),
            "user": item['properties'].pop(self.keys['user'], None),
            "geometry": GEOSGeometry(json.dumps(item['geometry'])),
            "elev": item['properties'].pop(self.keys['elev'], None),
            "description": item['properties'].pop(self.keys['description'], ''),
            "notes": item['properties'].pop(self.keys['notes'], ''),
            "added": item['properties'].pop(self.keys['added'], None),
            "updated": item['properties'].pop(self.keys['updated'], None),
            "data": {}
        }

        # loop over remainig items and put them into data dictionary
        for key, value in item['properties'].items():
            result["data"][key] = value

        return result



from nodeshot.core.base.managers import GeoPublishedQuerySet
from nodeshot.core.base.managers import HStoreGeoPublishedManager


class ExternalMixin(object):
    """ Add method external() to your custom queryset or manager model """
    
    def external(self):
        """ return only external layers """
        return self.filter(is_external=True)


class ExternalQueryset(GeoPublishedQuerySet, ExternalMixin):
    """ filter external layers """
    pass


class LayerManager(HStoreGeoPublishedManager, ExternalMixin):
    """ extends GeoPublishedManager to add external method """
    
    def get_query_set(self): 
        return ExternalQueryset(self.model, using=self._db)



"""
nodeshot.networking.links unit tests
"""

from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse

from nodeshot.core.base.tests import BaseTestCase
from nodeshot.core.base.tests import user_fixtures
from nodeshot.networking.net.models import Interface

from .models import Link
from .models.choices import LINK_STATUS, LINK_TYPES


class LinkTest(BaseTestCase):

    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json',
        'test_routing_protocols.json',
        'test_devices.json',
        'test_interfaces.json',
        'test_ip_addresses.json',
        'test_links'
    ]

    def setUp(self):
        l = Link()
        l.interface_a = Interface.objects.find(2)
        l.interface_b = Interface.objects.find(3)
        l.type = LINK_TYPES.get('radio')
        l.status = LINK_STATUS.get('active')
        self.link = l

    def test_non_radio_shouldnt_have_radio_info(self):
        """ *** A link of any type which is not "radio" should not have dBm or noise data *** """
        link = self.link
        link.type = LINK_TYPES.get('ethernet')
        link.dbm = -70
        link.noise = -90
        self.assertRaises(ValidationError, link.full_clean)

    def test_save_radio_link(self):
        """ *** It should be possible to save a new link *** """
        l = self.link
        l.type = LINK_TYPES.get('radio')
        l.dbm = -70
        l.noise = -90
        l.save()
        # delete variable l
        del l
        # retrieve link again from DB
        l = Link.objects.last()
        # check everything worked
        self.assertTrue(l.type == 1 and l.dbm == -70 and l.noise == -90, "something went wrong while saving a new link")

    def test_null_interfaces(self):
        """ *** An active link with null 'from interface' and 'to interface' fields should not be saved  *** """
        l = Link(type=LINK_TYPES.get('radio'), status=LINK_TYPES.get('active'))
        self.assertRaises(ValidationError, l.full_clean)

    def test_auto_fill_node_fields(self):
        """ *** When a link with any type except for 'planned' is saved it should automatically fill the fields 'from node' and 'to node'  *** """
        l = self.link
        l.type = LINK_TYPES.get('radio')
        l.save()
        self.assertTrue(l.node_a is not None and l.node_b is not None, '"from node" and "to node" fields are null')

    def test_null_interface_and_node_fields(self):
        """ *** It should not be possible to save a link which has void node and interface info  *** """
        link = Link(type=LINK_TYPES.get('radio'), status=LINK_STATUS.get('planned'))
        self.assertRaises(ValidationError, link.full_clean)

    def test_same_to_and_from_interface(self):
        link = self.link
        link.interface_a = Interface.objects.find(1)
        link.interface_b = Interface.objects.find(1)
        with self.assertRaises(ValidationError):
            link.full_clean()

        link2 = self.link
        link2.interface_b_id = 1
        link2.interface_a_id = 1
        with self.assertRaises(ValidationError):
            link.full_clean()

    def test_auto_linestring(self):
        link = self.link
        self.assertIsNone(link.line)
        link.save()
        self.assertIsNotNone(link.line)

    def test_additional_properties(self):
        link = self.link
        link.interface_b = Interface.objects.find(3)  # different node
        self.assertIsNone(link.node_a_name)
        self.assertIsNone(link.node_b_name)
        self.assertIsNone(link.node_a_slug)
        self.assertIsNone(link.node_b_slug)
        self.assertIsNone(link.interface_a_mac)
        self.assertIsNone(link.interface_b_mac)
        link.save()
        link = Link.objects.find(link.id)
        self.assertEqual(link.node_a_name, link.node_a.name)
        self.assertEqual(link.node_b_name, link.node_b.name)
        self.assertEqual(link.node_a_slug, link.node_a.slug)
        self.assertEqual(link.node_b_slug, link.node_b.slug)
        self.assertEqual(link.interface_a_mac, link.interface_a.mac)
        self.assertEqual(link.interface_b_mac, link.interface_b.mac)

    def test_link_interface_type(self):
        link = self.link
        link.interface_a = Interface.objects.find(1)  # ethernet, while interface_b is wireless

        with self.assertRaises(ValidationError):
            link.full_clean()

    def test_auto_link_type(self):
        link = self.link
        link.type = None
        link.save()
        link = Link.objects.find(link.id)
        self.assertEqual(link.type, LINK_TYPES.get('radio'))

    def test_links_api(self):
        link = self.link
        link.save()
        link = Link.objects.find(link.id)

        # GET: 200 - link list
        url = reverse('api_link_list')
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEqual(len(response.data['results']), 2)

        # GET: 200 - link list geojson
        url = reverse('api_links_geojson_list')
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEqual(len(response.data['features']), 2)

        # GET: 200 - link details
        url = reverse('api_link_details', args=[link.id])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

        # GET: 200 - link details geojson
        url = reverse('api_links_geojson_details', args=[link.id])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

    def test_node_links_api(self):
        link = self.link
        link.save()
        link = Link.objects.find(link.id)

        # GET: 200 - node A
        url = reverse('api_node_links', args=[link.node_a.slug])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(response.data), 2)

        # GET: 200 - node B
        url = reverse('api_node_links', args=[link.node_b.slug])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(response.data), 2)

        # GET: 404
        url = reverse('api_node_links', args=['idontexist'])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 404)



from __future__ import absolute_import

from django.contrib.gis.geos import GEOSGeometry
from django.utils.translation import ugettext_lazy as _
from .base import XMLParserMixin, GenericGisSynchronizer


class GeoRss(XMLParserMixin, GenericGisSynchronizer):
    """ Generic GeoRSS (simple version only) synchronizer """
    SCHEMA = [
        {
            'name': 'url',
            'class': 'CharField',
            'kwargs': {
                'help_text': _('URL containing geographical data'),
                'max_length': 128,
            }
        },
        {
            'name': 'verify_ssl',
            'class': 'BooleanField',
            'kwargs': {
                'default': True,
                'help_text': _('Wether the SSL certificates of the external services used should be verified or not')
            }
        },
        {
            'name': 'default_status',
            'class': 'CharField',
            'kwargs': {
                'blank': True,
                'max_length': 255,
                'help_text': _('Status for imported nodes, leave blank to use the default one')
            }
        },
        {
            'name': 'field_name',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'title',
                'verbose_name': _('name field'),
                'help_text': _('corresponding name field on external source')
            }
        },
        {
            'name': 'field_status',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'status',
                'verbose_name': _('status field'),
                'help_text': _('corresponding status field on external source')
            }
        },
        {
            'name': 'field_description',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': '',
                'blank': True,
                'verbose_name': _('description field'),
                'help_text': _('corresponding description field on external source')
            }
        },
        {
            'name': 'field_address',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default':
                'address',
                'verbose_name': _('address field'),
                'help_text': _('corresponding address field on external source')
            }
        },
        {
            'name': 'field_is_published',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'is_published',
                'verbose_name': _('is_published field'),
                'help_text': _('corresponding is_published field on external source')
            }
        },
        {
            'name': 'field_user',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'user',
                'verbose_name': _('user field'),
                'help_text': _('corresponding user field on external source')
            }
        },
        {
            'name': 'field_elev',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'elev',
                'verbose_name': _('elev field'),
                'help_text': _('corresponding elev field on external source')
            }
        },
        {
            'name': 'field_notes',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'notes',
                'verbose_name': _('notes field'),
                'help_text': _('corresponding notes field on external source')
            }
        },
        {
            'name': 'field_added',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'pubDate',
                'verbose_name': _('added field'),
                'help_text': _('corresponding added field on external source')
            }
        },
        {
            'name': 'field_updated',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'updated',
                'verbose_name': _('updated field'),
                'help_text': _('corresponding updated field on external source')
            }
        }
    ]

    def key_mapping(self, ):
        key_map = self.field_mapping

        if 'summary' in self.data:
            description_default_key = 'summary'
        else:
            description_default_key = 'description'

        self.keys = {
            "name": key_map.get('name', 'title'),
            "status": key_map.get('status', 'status'),
            "description": key_map.get('description', description_default_key),
            "address": key_map.get('address', 'address'),
            "is_published": key_map.get('is_published', 'is_published'),
            "user": key_map.get('user', 'user'),
            "elev": key_map.get('elev', 'elev'),
            "notes": key_map.get('notes', 'notes'),
            "added": key_map.get('added', 'pubDate'),
            "updated": key_map.get('updated', 'updated'),
        }
        self.default_status = self.config.get('default_status', '')

    def parse(self):
        """ parse data """
        super(GeoRss, self).parse()

        # support RSS and ATOM
        tag_name = 'item' if '<item>' in self.data else 'entry'

        self.parsed_data = self.parsed_data.getElementsByTagName(tag_name)

    def parse_item(self, item):
        try:
            lat, lng = self.get_text(item, 'georss:point').split(' ')
        except IndexError:
            try:
                # detail view
                lat = self.get_text(item, 'georss:lat')
                lng = self.get_text(item, 'georss:long')
            except IndexError:
                # W3C
                lat = self.get_text(item, 'geo:lat')
                lng = self.get_text(item, 'geo:long')

        result = {
            "name": self.get_text(item, self.keys['name']),
            "status": None,
            "address": '',
            "is_published": True,
            "user": None,
            "geometry": GEOSGeometry('POINT (%s %s)' % (lng, lat)),
            "elev": None,
            "description": self.get_text(item, self.keys['description'], ''),
            "notes": '',
            "added": self.get_text(item, self.keys['added'], None),
            "updated": self.get_text(item, self.keys['updated'], None),
            "data": {}
        }

        return result



from django.conf import settings
from django.utils.translation import ugettext_lazy as _


TEXTS = getattr(settings, 'NODESHOT_NOTIFICATIONS_TEXTS', {
    'custom': None,
    'node_created': _('A new node with name "%(name)s" has been created.'),
    'node_status_changed': _('Status of node "%(name)s" has changed from "%(old_status)s" to "%(new_status)s".'),
    'node_own_status_changed': _('Status of your node "%(name)s" changed from "%(old_status)s" to "%(new_status)s".'),
    'node_deleted': _('Node "%(name)s" with ID #%(id)s was deleted.'),
})
REGISTER = getattr(settings, 'NODESHOT_NOTIFICATIONS_REGISTER', ('nodeshot.community.notifications.registrars.nodes',))
# boolean: users can only turn on or off
# distance: users can turn off (-1), turn on for all (0) or set a range of km (n)
USER_SETTING = getattr(settings, 'NODESHOT_NOTIFICATIONS_USER_SETTING', {
    'node_created':             { 'type': 'distance', 'geo_field': 'geometry' },
    'node_status_changed':      { 'type': 'distance', 'geo_field': 'geometry' },
    'node_deleted':             { 'type': 'distance', 'geo_field': 'geometry' },
    'node_own_status_changed':  { 'type': 'boolean' },
})
DEFAULT_BOOLEAN = getattr(settings, 'NODESHOT_NOTIFICATIONS_DEFAULT_BOOLEAN', True)
DEFAULT_DISTANCE = getattr(settings, 'NODESHOT_NOTIFICATIONS_DEFAULT_DISTANCE', 30)
DELETE_OLD = getattr(settings, 'NODESHOT_NOTIFICATIONS_DELETE_OLD', 40)



from django.core.urlresolvers import NoReverseMatch

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.reverse import reverse

from .urls import urlpatterns


@api_view(('GET',))
def root_endpoint(request, format=None):
    """
    List of all the available resources of this RESTful API.
    """
    endpoints = []
    # loop over url modules
    for urlmodule in urlpatterns:
        # is it a urlconf module?
        if hasattr(urlmodule, 'urlconf_module'):
            is_urlconf_module = True
        else:
            is_urlconf_module = False
            
        # if url is really a urlmodule
        if is_urlconf_module:
            # loop over urls of that module
            for url in urlmodule.urlconf_module.urlpatterns:
                # TODO: configurable skip url in settings
                # skip api-docs url
                if url.name in ['django.swagger.resources.view']:
                    continue
                # try adding url to list of urls to show
                try:
                    endpoints.append({
                        'name': url.name.replace('api_', ''),
                        'url': reverse(url.name, request=request, format=format)
                    })
                # urls of object details will fail silently (eg: /nodes/<slug>/)
                except NoReverseMatch:
                    pass

    return Response(endpoints)



from django.shortcuts import render_to_response
from django.template import RequestContext

from . import DOMAIN, PORT  # contained in __init__.py


def test(request):
    context = {
        'DOMAIN': DOMAIN,
        'PORT': PORT
    }
    return render_to_response('test.html', context,
                              context_instance=RequestContext(request))


from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_page
from django.db.models import Q

from rest_framework import permissions, authentication, generics

from nodeshot.core.base.mixins import ACLMixin, CustomDataMixin
from nodeshot.core.base.utils import Hider

from .settings import REVERSION_ENABLED
from .permissions import IsOwnerOrReadOnly
from .serializers import *  # noqa
from .models import Node, Status, Image


if REVERSION_ENABLED:
    from nodeshot.core.base.mixins import RevisionCreate, RevisionUpdate

    class NodeListBase(ACLMixin, RevisionCreate, generics.ListCreateAPIView):
        pass

    class NodeDetailBase(ACLMixin, RevisionUpdate, generics.RetrieveUpdateDestroyAPIView):
        pass
else:
    class NodeListBase(ACLMixin, generics.ListCreateAPIView):
        pass

    class NodeDetailBase(ACLMixin, generics.RetrieveUpdateDestroyAPIView):
        pass


def get_queryset_or_404(queryset, kwargs):
    """
    Checks if object returned by queryset exists
    """
    # ensure exists
    try:
        obj = queryset.get(**kwargs)
    except Exception:
        raise Http404(_('Not found'))

    return obj


class NodeList(NodeListBase):
    """
    Retrieve list of all published nodes.

    Parameters:

     * `search=<word>`: search <word> in name, slug, description and address of nodes
     * `layers=<layer1>,<layer2>`: retrieve nodes of specified layers (comma separated)
     * `limit=<n>`: specify number of items per page (defaults to 50)

    ### POST

    Create a new node. Requires authentication.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    queryset = Node.objects.published()
    serializer_class = NodeListSerializer
    pagination_serializer_class = PaginatedNodeListSerializer
    paginate_by_param = 'limit'
    paginate_by = 50

    def pre_save(self, obj):
        """ automatically determine user on creation """
        if not obj.id:
            obj.user_id = self.request.user.id

    def get_queryset(self):
        """
        Optionally restricts the returned nodes
        by filtering against a `search` query parameter in the URL.
        """
        # retrieve all nodes which are published and accessible to current user
        # and use joins to retrieve related fields
        queryset = super(NodeList, self).get_queryset().select_related('layer', 'status', 'user')
        # query string params
        search = self.request.QUERY_PARAMS.get('search', None)
        layers = self.request.QUERY_PARAMS.get('layers', None)
        if search is not None:
            search_query = (
                Q(name__icontains=search) |
                Q(slug__icontains=search) |
                Q(description__icontains=search) |
                Q(address__icontains=search)
            )
            # add instructions for search to queryset
            queryset = queryset.filter(search_query)
        if layers is not None:
            # look for nodes that are assigned to the specified layers
            queryset = queryset.filter(Q(layer__slug__in=layers.split(',')))
        return queryset

node_list = NodeList.as_view()


class NodeDetail(NodeDetailBase):
    """
    Retrieve details of specified node. Node must be published and accessible.

    ### DELETE

    Delete specified nodes. Must be authenticated as owner or admin.

    ### PUT & PATCH

    Edit node. Must be authenticated as owner or admin.
    """
    lookup_field = 'slug'
    serializer_class = NodeDetailSerializer
    authentication_classes = (authentication.SessionAuthentication, )
    permission_classes = (IsOwnerOrReadOnly, )
    queryset = Node.objects.published().select_related('user', 'layer')

node_details = NodeDetail.as_view()


class NodeGeoJSONList(NodeList):
    """
    Retrieve list of all published nodes in GeoJSON format.

    Parameters:

     * `search=<word>`: search <word> in name, slug, description and address of nodes
     * `layers=<layer1>,<layer2>`: retrieve nodes of specified layers
     * `limit=<n>`: specify number of items per page (show all by default)
     * `page=<n>`: show page n
    """
    pagination_serializer_class = PaginatedGeojsonNodeListSerializer
    paginate_by_param = 'limit'
    paginate_by = 0
    serializer_class = NodeGeoSerializer
    post = Hider()

geojson_list = NodeGeoJSONList.as_view()


# -------- Images -------- #


class NodeImageList(CustomDataMixin, generics.ListCreateAPIView):
    """
    Retrieve a list of images of the specified node.
    Node must exist and be published.

    ### POST

    Upload a new image.

    **Permissions:** only owner of a node can upload images.

    **Fields**:

     * `file`: binary file of the image, accepted file types are JPEG, PNG and GIF - **required**
     * `description`: description / caption of the image
     * `order`: order of the image, leave blank to set as last
     * `access_level`: determines who can see the image, defaults to public

    ### How do I upload an image?

    Set the content-type as **"multipart-formdata"** and send the file param as a binary stream.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    serializer_class = ImageListSerializer
    serializer_custom_class = ImageAddSerializer
    permission_classes = (IsOwnerOrReadOnly, )

    def get_custom_data(self):
        """ additional request.DATA """
        return {
            'node': self.node.id
        }

    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure node exists and store it in an instance attribute
            * change queryset to return only images of current node
        """
        super(NodeImageList, self).initial(request, *args, **kwargs)

        # ensure node exists
        self.node = get_queryset_or_404(
            Node.objects.published().accessible_to(request.user),
            {'slug': self.kwargs.get('slug', None)}
        )

        # check permissions on node (for image creation)
        self.check_object_permissions(request, self.node)

        # return only images of current node
        self.queryset = Image.objects.filter(node_id=self.node.id).accessible_to(self.request.user).select_related('node')

node_images = NodeImageList.as_view()


class ImageDetail(ACLMixin, generics.RetrieveUpdateDestroyAPIView):
    """
    Retrieve details of specified image.

    ### DELETE

    Delete specified nodes. Must be authenticated as owner or admin.

    ### PUT & PATCH

    Edit image.

    **Permissions:** only owner of a node can edit.

    Example of **JSON** representation that should be sent:

        {
            "description": "image caption",
            "order": 3,
            "access_level": "public"
        }
    """
    queryset = Image.objects.all()
    serializer_class = ImageEditSerializer
    authentication_classes = (authentication.SessionAuthentication, )
    permission_classes = (IsOwnerOrReadOnly, )
    lookup_field = 'pk'

    def get_queryset(self):
        self.node = get_queryset_or_404(
            Node.objects.published().accessible_to(self.request.user),
            {'slug': self.kwargs.get('slug', None)}
        )

        return super(ImageDetail, self).get_queryset().filter(node=self.node)

node_image_detail = ImageDetail.as_view()


# --------- Status ---------#


class StatusList(generics.ListAPIView):
    """
    Retrieve a list of all the available statuses and their relative icons/colors.
    """
    queryset = Status.objects.all()
    serializer_class = StatusListSerializer

    @method_decorator(cache_page(86400))  # cache for 1 day
    def dispatch(self, *args, **kwargs):
        return super(StatusList, self).dispatch(*args, **kwargs)

status_list = StatusList.as_view()


# --------- Utils ---------#

import requests
from collections import OrderedDict
from django.contrib.gis.geos import Point, LineString
from rest_framework.decorators import api_view
from rest_framework.response import Response
from .settings import ELEVATION_API_KEY, ELEVATION_DEFAULT_SAMPLING


@api_view(('GET',))
def elevation_profile(request, format=None):
    """
    Proxy to google elevation API but returns GeoJSON
    (unless "original" parameter is passed, in which case the original response is returned).

    For input parameters read:
    https://developers.google.com/maps/documentation/elevation/
    """
    if format is None:
        format = 'json'
    url = 'https://maps.googleapis.com/maps/api/elevation/{0}'.format(format)
    params = request.QUERY_PARAMS.copy()
    original = 'original' in request.QUERY_PARAMS
    is_path = 'path' in request.QUERY_PARAMS
    samples = request.QUERY_PARAMS.get('samples')

    # add api key if present
    if ELEVATION_API_KEY:
        params['key'] = ELEVATION_API_KEY

    # automatic sampling
    if is_path and not samples:
        points = []
        # convert path in list of Point objects
        for latlng in request.QUERY_PARAMS['path'].split('|'):
            latlng = latlng.split(',')
            points.append(Point(float(latlng[1]), float(latlng[0])))
        if len(points) > 1:
            # length of the path in meters
            length = LineString(*points).length * 100000
            # get 1 point every x meters, where x is defined in ELEVATION_DEFAULT_SAMPLING
            samples = int(round(length / ELEVATION_DEFAULT_SAMPLING))
            # use the automatically calculated value as long as it is compatibile with the API usage limits
            params['samples'] = samples if samples <= 512 else 512
    # send request to Google Elevation API
    response = requests.get(url, params=params)
    data = response.json()

    # if ok convert to GeoJSON
    if data['status'] == 'OK' and original is False:
        # if more than one result use LineString
        if len(data['results']) > 1:
            geometry = 'LineString'
        # else use Point
        else:
            geometry = 'Point'
        # lng, lat, z coordinates
        coordinates = []
        for point in data['results']:
            coordinates.append([point['location']['lng'],
                                point['location']['lat'],
                                point['elevation']])
        return Response(OrderedDict((
            ('type', 'Feature'),
            ('geometry', OrderedDict((
                ('type', geometry),
                ('coordinates', coordinates)
            )))
        )))
    # else return original response
    else:
        return Response(data, status=response.status_code)



from django.contrib import admin
from django.db import models

from nodeshot.core.base.admin import BaseAdmin, BaseStackedInline
from nodeshot.core.base.widgets import AdvancedFileInput
from nodeshot.networking.net.admin import DeviceAdmin

from models import *


class MacPrefixInline(admin.StackedInline):
    model = MacPrefix
    extra = 0
    inline_classes = ('grp-collapse grp-open',)


class ManufacturerAdmin(BaseAdmin):
    list_display  = ('name', 'image_img_tag', 'url_tag', 'added', 'updated')
    list_display_links = ('name', 'image_img_tag')
    search_fields = ('name',)
    inlines = [MacPrefixInline]
    
    formfield_overrides = {
        models.ImageField: {'widget': AdvancedFileInput(image_width=250)},
    }


class DeviceModelAdmin(BaseAdmin):
    list_display  = ('name', 'image_img_tag', 'added', 'updated')
    list_display_links = ('name', 'image_img_tag')
    search_fields = ('name',)
    
    formfield_overrides = {
        models.ImageField: {'widget': AdvancedFileInput(image_width=250)},
    }


class RadiationPatternInline(BaseStackedInline):
    model = RadiationPattern
    extra = 0
    inline_classes = ('grp-collapse grp-open',)


class AntennaModelAdmin(BaseAdmin):
    list_display  = ('name', 'image_img_tag', 'added', 'updated')
    list_display_links = ('name', 'image_img_tag')
    search_fields = ('name',)
    inlines = [RadiationPatternInline]
    
    formfield_overrides = {
        models.ImageField: {'widget': AdvancedFileInput(image_width=250)},
    }


admin.site.register(Manufacturer, ManufacturerAdmin)
admin.site.register(DeviceModel, DeviceModelAdmin)
admin.site.register(AntennaModel, AntennaModelAdmin)


# Extend Default Device Admin

class DeviceToModelRelInline(admin.StackedInline):
    model = DeviceToModelRel
    inline_classes = ('grp-collapse grp-open',)
    
    raw_id_fields = ('model',)
    autocomplete_lookup_fields = {
        'fk': ['model'],
    }


class AntennaInline(BaseStackedInline):
    model = Antenna
    extra = 0
    inline_classes = ('grp-collapse grp-open',)
    
    raw_id_fields = ('model',)
    autocomplete_lookup_fields = {
        'fk': ['model'],
    }


DeviceAdmin.inlines.insert(0, DeviceToModelRelInline)
DeviceAdmin.inlines.insert(1, AntennaInline)



import json

from django.contrib import admin
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.admin import BaseAdmin
from .models import Metric


class MetricAdmin(BaseAdmin):
    list_display = ('name', 'tags_formatted', 'added', 'updated')
    list_filter = ('added', 'updated')
    search_fields = ('name',)
    actions_on_bottom = True
    ordering = ('-id',)
    change_form_template = 'admin/metric_change_form.html'

    def tags_formatted(self, obj):
        return json.dumps(obj.tags) if obj.tags else ''
    tags_formatted.short_description = _('tags')

    # define the autocomplete_lookup_fields
    if 'grappelli' in settings.INSTALLED_APPS:
        autocomplete_lookup_fields = {
            'generic': [['content_type', 'object_id']],
        }

    class Media:
        css = {
            'all': ('metrics/css/nv.d3.min.css',)
        }
        js = (
            'metrics/js/d3.min.js',
            'metrics/js/nv.d3.min.js',
            'metrics/js/graphs.js',
        )


admin.site.register(Metric, MetricAdmin)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.core.validators import MaxLengthValidator, MinLengthValidator
from django.core.exceptions import ValidationError
from django.core.mail import EmailMessage
from django.template.loader import render_to_string

from nodeshot.core.base.models import BaseDate

from .choices import INWARD_STATUS_CHOICES
from ..settings import settings, INWARD_REQUIRE_AUTH, INWARD_MAXLENGTH, INWARD_MINLENGTH, INWARD_LOG


user_app_label = settings.AUTH_USER_MODEL.split('.')[0]
user_model_name = settings.AUTH_USER_MODEL.split('.')[1]
limit = (
    models.Q(app_label='nodes', model='node') |
    models.Q(app_label=user_app_label, model=user_model_name.lower()) |
    models.Q(app_label='layers', model='layer')
)
USER_CAN_BE_BLANK = not INWARD_REQUIRE_AUTH


class Inward(BaseDate):
    """
    Incoming messages from users
    """
    # could be a node, an user or a layer
    status = models.IntegerField(_('status'), choices=INWARD_STATUS_CHOICES, default=INWARD_STATUS_CHOICES[1][0])
    content_type = models.ForeignKey(ContentType, limit_choices_to=limit)
    object_id = models.PositiveIntegerField()
    to = generic.GenericForeignKey('content_type', 'object_id')
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('user'), blank=USER_CAN_BE_BLANK, null=True)
    from_name = models.CharField(_('name'), max_length=50, blank=True)
    from_email = models.EmailField(_('email'), max_length=50, blank=True)
    message = models.TextField(_('message'), validators=[
        MaxLengthValidator(INWARD_MAXLENGTH),
        MinLengthValidator(INWARD_MINLENGTH)
    ])
    ip = models.GenericIPAddressField(verbose_name=_('ip address'), blank=True, null=True)
    user_agent = models.CharField(max_length=255, blank=True)
    accept_language = models.CharField(max_length=255, blank=True)

    class Meta:
        verbose_name = _('Inward message')
        verbose_name_plural = _('Inward messages')
        app_label = 'mailing'
        ordering = ['-status']

    def __unicode__(self):
        return _(u'Message from %(from)s to %(to)s') % ({'from': self.from_name, 'to': self.content_type})

    def clean(self, *args, **kwargs):
        """ custom validation """
        if not self.user and (not self.from_name or not self.from_email):
            raise ValidationError(_('If sender is not specified from_name and from_email must be filled in.'))
        # fill name and email
        if self.user:
            self.from_name = self.user.get_full_name()
            self.from_email = self.user.email

    def send(self):
        """
        Sends the email to the recipient
        If the sending fails will set the status of the instance to "error" and will log the error according to your project's django-logging configuration
        """
        if self.content_type.name == 'node':
            to = [self.to.user.email]
        elif self.content_type.name == 'layer':
            to = [self.to.email]
            # layer case is slightly special, mantainers need to be notified as well
            # TODO: consider making the mantainers able to switch off notifications
            for mantainer in self.to.mantainers.all().only('email'):
                to += [mantainer.email]
        else:
            to = [self.to.email]

        context = {
            'sender_name': self.from_name,
            'sender_email': self.from_email,
            'message': self.message,
            'site': settings.SITE_NAME,
            'object_type': self.content_type.name,
            'object_name': str(self.to)
        }
        message = render_to_string('mailing/inward_message.txt', context)

        email = EmailMessage(
            # subject
            _('Contact request from %(sender_name)s - %(site)s') % context,
            # message
            message,
            # from
            settings.DEFAULT_FROM_EMAIL,
            # to
            to,
            # reply-to header
            headers={'Reply-To': self.from_email}
        )

        import socket
        # try sending email
        try:
            email.send()
            self.status = 1
        # if error
        except socket.error as e:
            # log the error
            import logging
            log = logging.getLogger(__name__)
            error_msg = 'nodeshot.community.mailing.models.inward.send(): %s' % e
            log.error(error_msg)
            # set status of the instance as "error"
            self.status = -1

    def save(self, *args, **kwargs):
        """
        Custom save method
        """
        # fill name and email
        if self.user:
            self.from_name = self.user.get_full_name()
            self.from_email = self.user.email

        # if not sent yet
        if int(self.status) < 1:
            # tries sending email (will modify self.status!)
            self.send()

        # save in the database unless logging is explicitly turned off in the settings file
        if INWARD_LOG:
            super(Inward, self).save(*args, **kwargs)



import os

from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseOrderedACL
from nodeshot.core.base.managers import AccessLevelManager


class Image(BaseOrderedACL):
    """
    Images of a 'Node'
    """
    node = models.ForeignKey('nodes.Node', verbose_name=_('node'))
    file = models.ImageField(upload_to='nodes/', verbose_name=_('image'))
    description = models.CharField(_('description'), max_length=255, blank=True, null=True)

    # manager
    objects = AccessLevelManager()

    class Meta:
        db_table = 'nodes_image'
        app_label = 'nodes'
        permissions = (('can_view_image', 'Can view images'),)
        ordering = ['order']

    def __unicode__(self):
        return self.file.name

    def get_auto_order_queryset(self):
        """ overriding a BaseOrdered Abstract Model method """
        return self.__class__.objects.filter(node=self.node)

    def delete(self, *args, **kwargs):
        """ delete image when an image record is deleted """
        try:
            os.remove(self.file.file.name)
        # image does not exist
        except (OSError, IOError):
            pass

        super(Image, self).delete(*args, **kwargs)



from django.contrib import admin
from django.db import models

from nodeshot.core.base.admin import BaseGeoAdmin, BaseStackedInline, PublishActionsAdminMixin
from nodeshot.core.base.widgets import AdvancedFileInput

from .settings import settings, REVERSION_ENABLED, DESCRIPTION_HTML
from .models import Node, Status, Image


# enable django-reversion according to settings
if REVERSION_ENABLED:
    import reversion

    class GeoAdmin(BaseGeoAdmin, reversion.VersionAdmin):
        change_list_template = 'reversion_and_smuggler/change_list.html'
else:
    class GeoAdmin(BaseGeoAdmin):
        change_list_template = 'smuggler/change_list.html'


class ImageInline(BaseStackedInline):
    model = Image

    formfield_overrides = {
        models.ImageField: {'widget': AdvancedFileInput(image_width=250)},
    }

    if 'grappelli' in settings.INSTALLED_APPS:
        sortable_field_name = 'order'
        classes = ('grp-collapse grp-open', )


NODE_FILTERS = ['is_published', 'status', 'access_level', 'added', 'updated']
NODE_LIST_DISPLAY = ['name', 'user', 'status', 'access_level', 'is_published', 'added', 'updated']
NODE_FIELDS_LOOKEDUP = [
    'user__id', 'user__username',
    'status__id', 'status__name', 'status__is_default',
    'name', 'access_level', 'is_published', 'added', 'updated'
]

# include layer in filters if layers app installed
if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
    NODE_FILTERS.insert(0, 'layer')
    NODE_LIST_DISPLAY.insert(2, 'layer')
    NODE_FIELDS_LOOKEDUP += ['layer__id', 'layer__name']


class NodeAdmin(PublishActionsAdminMixin, GeoAdmin):
    list_display = NODE_LIST_DISPLAY
    list_filter = NODE_FILTERS
    list_select_related = True
    search_fields = ('name',)
    actions_on_bottom = True
    date_hierarchy = 'added'
    ordering = ('-id',)
    prepopulated_fields = {'slug': ('name',)}
    inlines = [ImageInline]

    # django-grappelli usability improvements
    raw_id_fields = ('layer', 'user')
    autocomplete_lookup_fields = {
        'fk': ('layer', 'user'),
    }

    def queryset(self, request):
        return super(NodeAdmin, self).queryset(request).select_related('user', 'layer', 'status')

    if DESCRIPTION_HTML:
        # enable editor for "node description" only
        html_editor_fields = ['description']


class StatusAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug', 'description', 'order', 'is_default')
    prepopulated_fields = {'slug': ('name',)}
    list_editable = ('order', )

    change_list_template = 'smuggler/change_list.html'


admin.site.register(Node, NodeAdmin)
admin.site.register(Status, StatusAdmin)



import uuid
import tornado.websocket


class WebSocketHandler(tornado.websocket.WebSocketHandler):
    """
    simple websocket server for bidirectional communication between client and server
    """
    
    # public means non authenticated
    # private means authenticated
    channels = {
        'public': {},
        'private': {}
    }
    
    def send_message(self, *args):
        """ alias to write_message """
        self.write_message(*args)
    
    def add_client(self, user_id=None):
        """
        Adds current instance to public or private channel.
        If user_id is specified it will be added to the private channel,
        If user_id is not specified it will be added to the public one instead.
        """
        if user_id is None:
            # generate a random uuid if it's an unauthenticated client
            self.channel = 'public'
            user_id = uuid.uuid1().hex
        else:
            self.channel = 'private'
        
        self.id = user_id
        self.channels[self.channel][self.id] = self
        print 'Client connected to the %s channel.' % self.channel
    
    def remove_client(self):
        """ removes a client """
        del self.channels[self.channel][self.id]
    
    @classmethod
    def broadcast(cls, message):
        """ broadcast message to all connected clients """
        clients = cls.get_clients()
        # loop over every client and send message
        for id, client in clients.iteritems():
            client.send_message(message)
    
    @classmethod
    def send_private_message(self, user_id, message):
        """
        Send a message to a specific client.
        Returns True if successful, False otherwise
        """
        try:
            client = self.channels['private'][str(user_id)]
        except KeyError:
            print '====debug===='
            print self.channels['private']
            print 'client with id %s not found' % user_id
            return False
        
        client.send_message(message)
        print 'message sent to client #%s' % user_id
        return True
    
    @classmethod
    def get_clients(self):
        """ return a merge of public and private clients """
        public = self.channels['public']
        private = self.channels['private']
        return dict(public.items() + private.items())
    
    def open(self):
        """ method which is called every time a new client connects """
        print 'Connection opened.'
        
        # retrieve user_id if specified
        user_id = self.get_argument("user_id", None)
        # add client to list of connected clients
        self.add_client(user_id)
        # welcome message
        self.send_message("Welcome to nodeshot websocket server.")
        # new client connected message
        client_count = len(self.get_clients().keys())
        new_client_message = 'New client connected, now we have %d %s!' % (client_count, 'client' if client_count <= 1 else 'clients')
        # broadcast new client connected message to all connected clients
        self.broadcast(new_client_message)
        
        print self.channels['private']

    def on_message(self, message):
        """ method which is called every time the server gets a message from a client """
        if message == "help":
            self.send_message("Need help, huh?")
        print 'Message received: \'%s\'' % message

    def on_close(self):
        """ method which is called every time a client disconnects """
        print 'Connection closed.'
        self.remove_client()
        
        client_count = len(self.get_clients().keys())
        new_client_message = '1 client disconnected, now we have %d %s!' % (client_count, 'client' if client_count <= 1 else 'clients')
        self.broadcast(new_client_message)



from celery import task
from .settings import PUBLIC_PIPE, PRIVATE_PIPE


@task
def send_message(message, pipe='public'):
    """
    writes message to pipe
    """
    if pipe not in ['public', 'private']:
        raise ValueError('pipe argument can be only "public" or "private"')
    else:
        pipe = pipe.upper()

    pipe_path = PUBLIC_PIPE if pipe == 'PUBLIC' else PRIVATE_PIPE

    # create file if it doesn't exist, append contents
    pipeout = open(pipe_path, 'a')

    pipeout.write('%s\n' % message)
    pipeout.close()



from django.conf.urls import patterns, url

urlpatterns = patterns('nodeshot.community.notifications.views',
    url(r'^account/notifications/$', 'notification_list', name='api_notification_list'),
    url(r'^account/notifications/(?P<pk>[0-9]+)/$', 'notification_detail', name='api_notification_detail'),
    
    # email settings
    url(r'^account/notifications/email-settings/$',
        'notification_email_settings',
        name='api_notification_email_settings'),
    
    # web settings
    url(r'^account/notifications/web-settings/$',
        'notification_web_settings',
        name='api_notification_web_settings'),
)



from rest_framework import permissions, authentication, generics

from nodeshot.core.base.mixins import ACLMixin

from .serializers import *
from .models import *


class ServiceList(ACLMixin, generics.ListCreateAPIView):
    """
    Retrieve service list according to user access level
    
    Parameters:
    
     * `limit=<n>`: specify number of items per page (defaults to 40)
     * `limit=0`: turns off pagination
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.DjangoModelPermissionsOrAnonReadOnly,)
    model = Service
    queryset = Service.objects.all()
    serializer_class = ServiceListSerializer
    pagination_serializer_class = PaginatedServiceSerializer
    paginate_by_param = 'limit'
    paginate_by = 40
    
service_list = ServiceList.as_view()


class ServiceDetail(ACLMixin, generics.RetrieveUpdateDestroyAPIView):
    """
    Retrieve service detail according to user access level
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.DjangoModelPermissionsOrAnonReadOnly,)
    model = Service
    queryset = Service.objects.all()
    serializer_class = ServiceDetailSerializer
    
service_details = ServiceDetail.as_view()


class CategoryList(generics.ListCreateAPIView):
    """
    List of service categories
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.DjangoModelPermissionsOrAnonReadOnly,)
    model = Category
    serializer_class = CategorySerializer
    
service_category_list = CategoryList.as_view()


class CategoryDetail(generics.RetrieveUpdateAPIView):
    """
    Category detail view
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.DjangoModelPermissionsOrAnonReadOnly,)
    model = Category
    serializer_class = CategorySerializer
    
service_category_details = CategoryDetail.as_view()



from netfields import InetAddressField, CidrAddressField

from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from nodeshot.core.base.models import BaseAccessLevel
from ..managers import NetAccessLevelManager
from choices import IP_PROTOCOLS



class Ip(BaseAccessLevel):
    """ IP Address Model """
    interface = models.ForeignKey('net.Interface', verbose_name=_('interface'))
    address = InetAddressField(verbose_name=_('ip address'), unique=True, db_index=True)
    protocol = models.CharField(_('IP Protocol Version'), max_length=4, choices=IP_PROTOCOLS, default=IP_PROTOCOLS[0][0], blank=True)
    netmask = CidrAddressField(_('netmask (CIDR, eg: 10.40.0.0/24)'), blank=True, null=True)
    
    objects = NetAccessLevelManager()
    
    class Meta:
        app_label = 'net'
        permissions = (('can_view_ip', 'Can view ip'),)
        verbose_name = _('ip address')
        verbose_name_plural = _('ip addresses')
    
    def __unicode__(self):
        return '%s: %s' % (self.protocol, self.address)
    
    def clean(self, *args, **kwargs):
        """ TODO """
        # netaddr.IPAddress('10.40.2.1') in netaddr.IPNetwork('10.40.0.0/24')
        pass
    
    def save(self, *args, **kwargs):
        """
        Determines ip protocol version automatically.
        Stores address in interface shortcuts for convenience.
        """
        self.protocol = 'ipv%d' % self.address.version
        # save
        super(Ip, self).save(*args, **kwargs)
        
        # save shortcut on interfaces
        ip_cached_list = self.interface.ip_addresses
        # if not present in interface shorctus add it to the list
        if str(self.address) not in ip_cached_list:
            # recalculate cached_ip_list
            recalculated_ip_cached_list = []
            for ip in self.interface.ip_set.all():
                recalculated_ip_cached_list.append(str(ip.address))
            # rebuild string in format "<ip_1>, <ip_2>"
            self.interface.ip_addresses = recalculated_ip_cached_list
            self.interface.save()
    
    @property
    def owner(self):
        return self.interface.owner
    
    if 'grappelli' in settings.INSTALLED_APPS:
        @staticmethod
        def autocomplete_search_fields():
            return ('address__icontains',)



from django.conf import settings


REGISTRATION_OPEN = getattr(settings, 'NODESHOT_PROFILES_REGISTRATION_OPEN', True)
EMAIL_CONFIRMATION = getattr(settings, 'NODESHOT_PROFILES_EMAIL_CONFIRMATION', True)
REQUIRED_FIELDS = getattr(settings, 'NODESHOT_PROFILES_REQUIRED_FIELDS', ['email'])



import os

from django.contrib import admin
from django.conf import settings

from nodeshot.core.base.admin import BaseAdmin, BaseStackedInline
from .models import DeviceConnector


class DeviceConnectorAdmin(BaseAdmin):
    list_display  = (
        'host', 'device', 'node',
        'port', 'backend', 'added', 'updated'
    )
    list_filter   = ('backend', 'added')
    date_hierarchy = 'added'
    ordering = ('-id',)
    search_fields = ('host', 'data')
    
    if 'grappelli' in settings.INSTALLED_APPS:
        raw_id_fields = ('node', 'device')
        autocomplete_lookup_fields = {
            'fk': ('node', 'device'),
        }
        
        change_form_template = 'admin/device_connector_customization.html'

admin.site.register(DeviceConnector, DeviceConnectorAdmin)


# ------ Extend Default Device Admin ------ #


from nodeshot.networking.net.models import Device
from nodeshot.networking.net.admin import DeviceAdmin as BaseDeviceAdmin


class DeviceConnectorInline(BaseStackedInline):
    model = DeviceConnector
    extra = 0
    sortable_field_name = 'order'
    exclude = ['node']


class DeviceAdmin(BaseDeviceAdmin):
    """
    add DeviceConnectorInline to inlines of DeviceAmin
    """
    inlines = BaseDeviceAdmin.inlines + [DeviceConnectorInline]


# unregister BaseDeviceAdmin
admin.site.unregister(Device)
admin.site.register(Device, DeviceAdmin)



"""
This file was generated with the customdashboard management command and
contains the class for the main dashboard.

To activate your index dashboard add the following to your settings.py::
    GRAPPELLI_INDEX_DASHBOARD = 'nodeshot.dashboard.CustomIndexDashboard'
"""

from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings

from grappelli.dashboard import modules, Dashboard
from grappelli.dashboard.utils import get_admin_site_name

USER_APPS = ['django.contrib.*']
if 'nodeshot.community.profiles' in settings.INSTALLED_APPS:
    USER_APPS.append('nodeshot.community.profiles.*')


class NodeshotDashboard(Dashboard):
    """
    Custom index dashboard for www.
    """

    def init_with_context(self, context):
        site_name = get_admin_site_name(context)

        self.children.append(modules.AppList(
            _('Nodeshot Core'),
            collapsible=True,
            column=1,
            models=('nodeshot.core.*',),
        ))

        self.children.append(modules.AppList(
            _('Nodeshot Networking'),
            collapsible=True,
            column=1,
            models=('nodeshot.networking.*',),
        ))

        self.children.append(modules.AppList(
            _('Nodeshot Community'),
            collapsible=True,
            column=1,
            models=('nodeshot.community.*',),
        ))

        self.children.append(modules.AppList(
            _('Administration'),
            collapsible=True,
            column=2,
            models=USER_APPS,
        ))

        # append a recent actions module
        self.children.append(modules.RecentActions(
            _('Recent Actions'),
            limit=5,
            collapsible=False,
            column=2,
        ))

        if 'nodeshot.ui.default' in settings.INSTALLED_APPS:
            self.children.append(modules.LinkList(
                _('Frontend site'),
                column=3,
                children=[
                    {
                        'title': _('Go to Frontend Site'),
                        'url': reverse('ui:index'),
                        'external': False,
                    },
                ]
            ))

        if 'smuggler' in settings.INSTALLED_APPS:
            self.children.append(modules.LinkList(
                _('Backup & Restore data'),
                column=3,
                children=[
                    {
                        'title': _('Backup data as JSON file'),
                        'url': reverse('dump-data'),
                        'external': False,
                    },
                    {
                        'title': _('Restore JSON backup'),
                        'url': reverse('load-data'),
                        'external': False,
                    },
                ]
            ))

        if 'rosetta' in settings.INSTALLED_APPS:
            self.children.append(modules.LinkList(
                _('Translations'),
                column=3,
                children=[
                    {
                        'title': _('Manage translations'),
                        'url': reverse('rosetta-home'),
                        'external': False,
                    }
                ]
            ))

        if 'nodeshot.core.api' in settings.INSTALLED_APPS:
            self.children.append(modules.LinkList(
                _('API'),
                column=3,
                children=[
                    {
                        'title': _('Browsable API'),
                        'url': reverse('api_root_endpoint'),
                        'external': False,
                    },
                    {
                        'title': _('Swagger API docs'),
                        'url': reverse('django.swagger.base.view'),
                        'external': False,
                    },
                ]
            ))

        # append another link list module for "support".
        self.children.append(modules.LinkList(
            _('Media Management'),
            column=3,
            children=[
                {
                    'title': _('FileBrowser'),
                    'url': reverse('filebrowser:fb_browse'),
                    'external': False,
                },
            ]
        ))

        # append another link list module for "support".
        self.children.append(modules.LinkList(
            _('Support'),
            column=3,
            children=[
                {
                    'title': _('Nodeshot Documentation'),
                    'url': 'http://nodeshot.rtfd.org/',
                    'external': True,
                },
                {
                    'title': _('Django Documentation'),
                    'url': 'http://docs.djangoproject.com/',
                    'external': True,
                },
                {
                    'title': _('Grappelli Documentation'),
                    'url': 'http://packages.python.org/django-grappelli/',
                    'external': True,
                },
            ]
        ))



import re

from django.utils.translation import ugettext_lazy as _

from nodeshot.core.nodes.models import Node
from nodeshot.community.participation.models import Vote, Comment, Rating

from .settings import DISCOVERY


SERVICES = {
    'node': {
        'name': _('Node insertion'),
        'description': _('Insert new nodes'),
        'keywords': '',
        'group': ''
    },
    'vote': {
        'name': _('Vote'),
        'description': _('Like or Dislike something'),
        'keywords': '',
        'group': ''
    },
    'comment': {
        'name': _('Comment'),
        'description': _('Leave a comment'),
        'keywords': '',
        'group': ''
    },
    'rate': {
        'name': _('Rate'),
        'description': _('Leave your rating about something'),
        'keywords': '',
        'group': ''
    }
}

MODELS = {
    'node': Node,
    'vote': Vote,
    'comment': Comment,
    'rate': Rating,
}

iso8601_REGEXP = re.compile("^(-?(?:[1-9][0-9]*)?[0-9]{4})-(1[0-2]|0[1-9])-(3[0-1]|0[1-9]|[1-2][0-9])?T(2[0-3]|[0-1][0-9]):([0-5][0-9]):([0-5][0-9])(\.[0-9]+)??(Z|[+-](?:2[0-3]|[0-1][0-9]):[0-5][0-9])?$")



# part of the code of this app is based on pinax.account

from ..settings import EMAIL_CONFIRMATION


from .profile import Profile
from .social_link import SocialLink
from .password_reset import PasswordReset
from .emailconfirmation import EmailAddress, EmailConfirmation

__all__ = [
    'Profile',
    'SocialLink',
    'PasswordReset',
    'EmailAddress',
    'EmailConfirmation'
]

# ------ SIGNALS ------ #

# activate user on email confirmation

from django.dispatch import receiver

if EMAIL_CONFIRMATION:
    from ..signals import email_confirmed

    @receiver(email_confirmed, sender=EmailConfirmation)
    def activate_user(sender, email_address, **kwargs):
        """
        activate user when primary email is confirmed
        """
        user = email_address.user
        if user.is_active is False:
            user.is_active = True
            user.save()



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from choices import APPLICATION_PROTOCOLS, TRANSPORT_PROTOCOLS


class Url(BaseDate):
    service = models.ForeignKey('services.Service',
                                verbose_name=_('service'))
    transport = models.CharField(_('transport protocol'), max_length=5,
                                 choices=TRANSPORT_PROTOCOLS,
                                 default=TRANSPORT_PROTOCOLS[1][0])
    application = models.CharField(_('application protocol'), max_length=20,
                                   choices=APPLICATION_PROTOCOLS)
    ip = models.ForeignKey('net.Ip', verbose_name=_('ip address'))
    port = models.IntegerField(_('port'), blank=True, null=True)
    path = models.CharField(_('path'), max_length=50, blank=True)
    domain = models.CharField(_('domain'), max_length=50, blank=True)
    
    class Meta:
        app_label= 'services'
        db_table = 'service_urls'
        verbose_name = _('url')
        verbose_name_plural = _('urls')
    
    def __unicode__(self):
        value = ''
        if self.application:
            value = '%s://' % self.application
        if self.domain:
            value += self.domain
        else:
            if self.ip.protocol == 'ipv4':
                encaps = '%s'
            else:
                encaps = '[%s]'
            value += encaps % self.ip.address
        if self.port:
            value += ':%s' % self.port
        return value



import hashlib
import copy

from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import authenticate

from rest_framework import serializers
from rest_framework.reverse import reverse

from nodeshot.core.base.serializers import ExtraFieldSerializer, HyperlinkedField
from .models import Profile as User, PasswordReset, SocialLink
from .settings import settings, EMAIL_CONFIRMATION

PASSWORD_MAX_LENGTH = User._meta.get_field('password').max_length
NOTIFICATIONS_INSTALLED = 'nodeshot.community.notifications' in settings.INSTALLED_APPS

if EMAIL_CONFIRMATION:
    from .models import EmailAddress


__all__ = [
    'LoginSerializer',
    'ProfileSerializer',
    'ProfileOwnSerializer',
    'ProfileCreateSerializer',
    'ProfileRelationSerializer',
    'AccountSerializer',
    'ChangePasswordSerializer',
    'ResetPasswordSerializer',
    'ResetPasswordKeySerializer',
    'SocialLinkSerializer',
    'SocialLinkAddSerializer'
]

# email addresses
if EMAIL_CONFIRMATION:
    __all__ += [
        'EmailSerializer',
        'EmailAddSerializer',
        'EmailEditSerializer'
    ]

    class EmailSerializer(serializers.ModelSerializer):
        details = serializers.HyperlinkedIdentityField(lookup_field='pk', view_name='api_account_email_detail')
        resend_confirmation = serializers.SerializerMethodField('get_resend_confirmation')

        def get_resend_confirmation(self, obj):
            """ return resend_confirmation url """
            if obj.verified:
                return False
            request = self.context.get('request', None)
            format = self.context.get('format', None)
            return reverse('api_account_email_resend_confirmation',
                           args=[obj.pk], request=request, format=format)

        class Meta:
            model = EmailAddress
            fields = ('id', 'email', 'verified', 'primary', 'details', 'resend_confirmation')
            read_only_fields = ('verified', 'primary')

    # noqa
    class EmailAddSerializer(serializers.ModelSerializer):
        class Meta:
            model = EmailAddress
            read_only_fields = ('verified', 'primary')

    # noqa
    class EmailEditSerializer(EmailSerializer):
        def validate_primary(self, attrs, source):
            """
            primary field validation
            """
            primary = attrs[source]
            verified = self.object.verified

            if primary is True and verified is False:
                raise serializers.ValidationError(_('Email address cannot be made primary if it is not verified first'))

            if primary is False and verified is True:
                primary_addresses = EmailAddress.objects.filter(user=self.object.user, primary=True)

                if primary_addresses.count() == 1 and primary_addresses[0].pk == self.object.pk:
                    raise serializers.ValidationError(_('You must have at least one primary address.'))

            return attrs

        class Meta:
            model = EmailAddress
            fields = ('id', 'email', 'verified', 'primary', 'resend_confirmation')
            read_only_fields = ('verified', 'email')


class LoginSerializer(serializers.Serializer):
    username = serializers.CharField(max_length=User._meta.get_field('username').max_length)
    password = serializers.CharField(max_length=PASSWORD_MAX_LENGTH)
    remember = serializers.BooleanField(default=True, help_text=_("If checked you will stay logged in for 3 weeks"))

    def user_credentials(self, attrs):
        """
        Provides the credentials required to authenticate the user for login.
        """
        credentials = {}
        credentials["username"] = attrs["username"]
        credentials["password"] = attrs["password"]
        return credentials

    def validate(self, attrs):
        """ checks if login credentials are correct """
        user = authenticate(**self.user_credentials(attrs))

        if user:
            if user.is_active:
                self.instance = user
            else:
                raise serializers.ValidationError(_("This account is currently inactive."))
        else:
            error = _("Ivalid login credentials.")
            raise serializers.ValidationError(error)
        return attrs


class SocialLinkSerializer(serializers.ModelSerializer):
    user = serializers.Field(source='user.username')
    details = serializers.SerializerMethodField('get_detail_url')

    def get_detail_url(self, obj):
        """ return detail url """
        request = self.context.get('request', None)
        format = self.context.get('format', None)
        args = [obj.user.username, obj.pk]
        return reverse('api_user_social_links_detail', args=args, request=request, format=format)

    class Meta:
        model = SocialLink
        fields = ('id', 'user', 'url', 'description', 'added', 'updated', 'details')
        read_only_fields = ('added', 'updated',)


class SocialLinkAddSerializer(serializers.ModelSerializer):
    class Meta:
        model = SocialLink
        read_only_fields = ('added', 'updated', )


class ProfileSerializer(serializers.ModelSerializer):
    """ Profile Serializer for visualization """
    details = serializers.HyperlinkedIdentityField(lookup_field='username', view_name='api_profile_detail')
    avatar = serializers.SerializerMethodField('get_avatar')
    full_name = serializers.SerializerMethodField('get_full_name')
    location = serializers.SerializerMethodField('get_location')
    social_links_url = serializers.HyperlinkedIdentityField(lookup_field='username', view_name='api_user_social_links_list')
    social_links = SocialLinkSerializer(source='sociallink_set', many=True, read_only=True)

    if 'nodeshot.core.nodes' in settings.INSTALLED_APPS:
        nodes = serializers.HyperlinkedIdentityField(view_name='api_user_nodes', lookup_field='username')

    def get_avatar(self, obj):
        """ avatar from gravatar.com """
        return 'https://www.gravatar.com/avatar/%s' % hashlib.md5(obj.email).hexdigest()

    def get_full_name(self, obj):
        """ user's full name """
        return obj.get_full_name()

    def get_location(self, obj):
        """ return user's location """
        if not obj.city and not obj.country:
            return None
        elif obj.city and obj.country:
            return '%s, %s' % (obj.city, obj.country)
        elif obj.city or obj.country:
            return obj.city or obj.country

    class Meta:
        model = User
        fields = [
            'details', 'id',
            'username', 'full_name', 'first_name', 'last_name',
            'about', 'gender', 'birth_date', 'address', 'city',
            'country', 'location',
            'date_joined', 'last_login', 'avatar',
        ]

        if 'nodeshot.core.nodes' in settings.INSTALLED_APPS:
            fields.append('nodes')

        fields += ['social_links_url', 'social_links']

        read_only_fields = [
            'username',
            'date_joined',
            'last_login'
        ]


class ProfileOwnSerializer(ProfileSerializer):
    """ same as ProfileSerializer, with is_staff attribute """
    if EMAIL_CONFIRMATION:
        email_addresses = EmailSerializer(source='email_set', many=True, read_only=True)

    class Meta:
        model = User
        fields = copy.copy(ProfileSerializer.Meta.fields)
        if EMAIL_CONFIRMATION:
            fields.append('email_addresses')
        fields.append('is_staff')
        read_only_fields = copy.copy(ProfileSerializer.Meta.read_only_fields)
        read_only_fields.append('is_staff')


class ProfileCreateSerializer(ExtraFieldSerializer):
    """ Profile Serializer for User Creation """
    password_confirmation = serializers.CharField(label=_('password_confirmation'),
                                                  max_length=PASSWORD_MAX_LENGTH)
    email = serializers.CharField(source='email', required='email' in User.REQUIRED_FIELDS)

    def validate_password_confirmation(self, attrs, source):
        """
        password_confirmation check
        """
        password_confirmation = attrs[source]
        password = attrs['password']

        if password_confirmation != password:
            raise serializers.ValidationError(_('Password confirmation mismatch'))

        return attrs

    class Meta:
        model = User
        fields = (
            'id',
            # required
            'username', 'email', 'password', 'password_confirmation',
            # optional
            'first_name', 'last_name', 'about', 'gender',
            'birth_date', 'address', 'city', 'country'
        )
        non_native_fields = ('password_confirmation', )


class ProfileRelationSerializer(ProfileSerializer):
    """ Profile Serializer used for linking """
    class Meta:
        model = User
        fields = ('id', 'username', 'full_name', 'city', 'country', 'avatar', 'details')


# ------ Add user info to ExtensibleNodeSerializer ------ #

from nodeshot.core.nodes.base import ExtensibleNodeSerializer

ExtensibleNodeSerializer.add_relationship(
    name='user',
    serializer=ProfileRelationSerializer,
    queryset=lambda obj, request: obj.user
)


class AccountSerializer(serializers.ModelSerializer):
    """ Account serializer """
    profile = serializers.HyperlinkedIdentityField(
        lookup_field='username',
        view_name='api_profile_detail'
    )
    social_links = serializers.HyperlinkedIdentityField(
        lookup_field='username',
        view_name='api_user_social_links_list'
    )
    change_password = HyperlinkedField(
        view_name='api_account_password_change'
    )
    logout = HyperlinkedField(view_name='api_account_logout')

    if EMAIL_CONFIRMATION:
        email_addresses = HyperlinkedField(view_name='api_account_email_list')

    if NOTIFICATIONS_INSTALLED:
        web_notification_settings = HyperlinkedField(
            view_name='api_notification_web_settings'
        )
        email_notification_settings = HyperlinkedField(
            view_name='api_notification_email_settings'
        )

    class Meta:
        model = User
        fields = ['profile', 'social_links', 'change_password', 'logout']

        if EMAIL_CONFIRMATION:
            fields += ['email_addresses']

        if NOTIFICATIONS_INSTALLED:
            fields += ['web_notification_settings', 'email_notification_settings']


class ChangePasswordSerializer(serializers.Serializer):
    """
    Change password serializer
    """
    current_password = serializers.CharField(
        help_text=_('Current Password'),
        max_length=PASSWORD_MAX_LENGTH,
        required=False  # optional because users subscribed from social network won't have a password set
    )
    password1 = serializers.CharField(
        help_text=_('New Password'),
        max_length=PASSWORD_MAX_LENGTH
    )
    password2 = serializers.CharField(
        help_text=_('New Password (confirmation)'),
        max_length=PASSWORD_MAX_LENGTH
    )

    def validate_current_password(self, attrs, source):
        """
        current password check
        """
        if self.object and self.object.has_usable_password() and not self.object.check_password(attrs.get("current_password")):
            raise serializers.ValidationError(_('Current password is not correct'))
        return attrs

    def validate_password2(self, attrs, source):
        """
        password_confirmation check
        """
        password_confirmation = attrs[source]
        password = attrs['password1']

        if password_confirmation != password:
            raise serializers.ValidationError(_('Password confirmation mismatch'))

        return attrs

    def restore_object(self, attrs, instance=None):
        """ change password """
        if instance is not None:
            instance.change_password(attrs.get('password2'))
            return instance
        return None


class ResetPasswordSerializer(serializers.Serializer):
    email = serializers.EmailField(required=True)

    def validate_email(self, attrs, source):
        """ ensure email is in the database """
        if EMAIL_CONFIRMATION:
            condition = EmailAddress.objects.filter(email__iexact=attrs["email"], verified=True).count() == 0
        else:
            condition = User.objects.get(email__iexact=attrs["email"], is_active=True).count() == 0

        if condition is True:
            raise serializers.ValidationError(_("Email address not found"))

        return attrs

    def restore_object(self, attrs, instance=None):
        """ create password reset for user """
        password_reset = PasswordReset.objects.create_for_user(attrs["email"])
        password_reset.email = attrs["email"]

        return password_reset


class ResetPasswordKeySerializer(serializers.Serializer):
    password1 = serializers.CharField(
        help_text=_('New Password'),
        max_length=PASSWORD_MAX_LENGTH
    )
    password2 = serializers.CharField(
        help_text=_('New Password (confirmation)'),
        max_length=PASSWORD_MAX_LENGTH
    )

    def validate_password2(self, attrs, source):
        """
        password2 check
        """
        password_confirmation = attrs[source]
        password = attrs['password1']

        if password_confirmation != password:
            raise serializers.ValidationError(_('Password confirmation mismatch'))

        return attrs

    def restore_object(self, attrs, instance):
        """ change password """
        user = instance.user
        user.set_password(attrs["password1"])
        user.save()
        # mark password reset object as reset
        instance.reset = True
        instance.save()

        return instance



from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies=[
        'nodeshot.core.nodes',
        'nodeshot.core.layers',
        'nodeshot.community.participation'
    ],
    module='nodeshot.interop.open311'
)



from django.dispatch import Signal


user_logged_in = Signal(providing_args=["request", "user"])
password_changed = Signal(providing_args=["user"])

email_confirmed = Signal(providing_args=["email_address"])
email_confirmation_sent = Signal(providing_args=["confirmation"])



import os
import simplejson as json

from django.core.urlresolvers import reverse
from django.contrib.gis.geos import GEOSGeometry

from nodeshot.core.base.tests import BaseTestCase, user_fixtures

from .serializers import *
from .base import SERVICES


class Open311Request(BaseTestCase):

    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json',
        'test_images.json'
    ]

    def test_service_discovery(self):
        response = self.client.get(reverse('api_service_discovery'))
        # ensure 4 keys in response
        self.assertEqual(len(response.data), 4)

        # ensure 5 keys for each endpoint
        for endpoint in response.data['endpoints']:
            self.assertEqual(len(endpoint), 5)

    def test_service_list(self):
        response = self.client.get(reverse('api_service_definition_list'))

        # ensure 4 services
        self.assertEqual(len(response.data), 4)

        # ensure these words are in response
        for word in SERVICES.keys():
            self.assertContains(response, word)

    def test_node_service_definition(self):
        response = self.client.get(reverse('api_service_definition_detail', args=['node']))

        # ensure correct service code
        self.assertEqual(response.data['service_code'], 'node')

        # ensure 7 attributes
        self.assertEqual(len(response.data['attributes']), 8)

    def test_vote_service_definition(self):
        response = self.client.get(reverse('api_service_definition_detail', args=['vote']))

        # ensure correct service code
        self.assertEqual(response.data['service_code'], 'vote')

        # ensure 7 attributes
        self.assertEqual(len(response.data['attributes']), 2)

    def test_comment_service_definition(self):
        response = self.client.get(reverse('api_service_definition_detail', args=['comment']))

        # ensure correct service code
        self.assertEqual(response.data['service_code'], 'comment')

        # ensure 7 attributes
        self.assertEqual(len(response.data['attributes']), 2)

    def test_rate_service_definition(self):
        response = self.client.get(reverse('api_service_definition_detail', args=['rate']))

        # ensure correct service code
        self.assertEqual(response.data['service_code'], 'rate')

        # ensure 7 attributes
        self.assertEqual(len(response.data['attributes']), 2)

    def test_service_request_wrong_service(self):
        # check wrong service in service_request
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':'not_exists'}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)
        self.assertEqual(response.status_code, 404)

    def test_service_request_node(self):
        # service_request for nodes
        service_request={
                        'service_code':"node",
                        "name": "montesacro4",
                        "slug": "montesacro4",
                        "layer": "rome",
                        "lat": "22.5253",
                        "long": "41.8890",
                        "description": "test",
                        }
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)
        # Not authenticated : 403
        self.assertEqual(response.status_code, 403)
        login = self.client.login(username='admin', password='tester')
        # Authenticated users can insert service requests: 201
        response = self.client.post(url,service_request)
        self.assertEqual(response.status_code, 201)

    def test_service_request_node_incomplete_key(self):
        # POST requests

        # incorrect service_request for nodes
        login = self.client.login(username='admin', password='tester')

        service_request={
                        'service_code':"node",
                        "slug": "montesacro4",
                        "layer": "rome",
                        "lat": "22.5253",
                        "long": "41.8890",
                        "description": "test",
                        }
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)
        self.assertEqual(response.status_code, 400)

    def test_service_request_node_incomplete_value(self):
        # POST requests

        # incorrect service_request for nodes
        login = self.client.login(username='admin', password='tester')

        service_request={
                        'service_code':"node",
                        "name": "",
                        "slug": "montesacro4",
                        "layer": "rome",
                        "lat": "22.5253",
                        "long": "41.8890",
                        "description": "test",
                        }
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)
        self.assertEqual(response.status_code, 400)

    def test_service_request_vote(self):
        # service_request for votes
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':"vote","node": 1,"vote":1}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)
        self.assertEqual(response.status_code, 201)

    def test_service_request_vote_incomplete(self):
        # incomplete service_request for votes
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':"vote","vote":1}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)

        self.assertEqual(response.status_code, 400)

    def test_service_request_vote_incorrect(self):
        # incorrect service_request for votes
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':"vote","node": 1,"vote":10}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)
        self.assertEqual(response.status_code, 400)

    def test_service_request_rating(self):
        # service_request for rating
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':"rate","node": 1,"value":1}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)

        self.assertEqual(response.status_code, 201)

    def test_service_request_rating_incomplete(self):
        # incomplete service_request for rating
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':"rate","value":1}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)

        self.assertEqual(response.status_code, 400)

    def test_service_request_rating_incorrect(self):
        # incorrect service_request for rating
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':"rate","node": 1,"value":20}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)

        self.assertEqual(response.status_code, 400)

    def test_service_request_comment(self):
        # service_request for comments
        login = self.client.login(username='admin', password='tester')
        service_request={'service_code':"comment","node": 1,"text":"OK"}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)

        self.assertEqual(response.status_code, 201)

    def test_service_request_comment_incomplete(self):
        # incomplete service_request for comments
        login = self.client.login(username='admin', password='tester')
        # Vote
        service_request={'service_code':"comment","text":"OK"}
        url = "%s" % reverse('api_service_request_list')
        response = self.client.post(url,service_request)

        self.assertEqual(response.status_code, 400)

    def test_get_service_request(self):
        # GET request detail
        response = self.client.get(reverse('api_service_request_detail', args=['node','1']))
        self.assertEqual(response.status_code, 200)
        # Wrong service code
        response = self.client.get(reverse('api_service_request_detail', args=['wrong','1']))
        self.assertEqual(response.status_code, 404)
        # Not existing request
        response = self.client.get(reverse('api_service_request_detail', args=['node','100']))
        self.assertEqual(response.status_code, 404)

    def test_get_service_requests(self):
        # GET requests list
        response = self.client.get(reverse('api_service_request_list'))
        self.assertEqual(response.status_code, 404)

        parameters = {'service_code': 'node'}
        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 10)

        parameters['status'] = 'open'
        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 3)

        parameters['status'] = 'closed'
        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 7)

        parameters['status'] = 'wrong'
        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 404)

        # check date parameters
        parameters = {'service_code':'node','start_date':'wrong'}

        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 404)

        parameters['start_date'] = '2013-01-01T17:57:02Z'
        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 10)

        parameters['end_date'] = '2013-04-01T17:57:02Z'
        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0)

        parameters['start_date'] = '2015-01-01T17:57:02Z'
        response = self.client.get(reverse('api_service_request_list'), parameters)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0)

    def test_file_upload(self):
        login = self.client.login(username='admin', password='tester')
        service_request={
                        'service_code':"node",
                        "name": "montesacro4",
                        "slug": "montesacro4",
                        "layer": "rome",
                        "lat": "22.5253",
                        "long": "41.8890",
                        "description": "test",
                        "geometry": "POINT (22.5253334454477372 41.8890404543067518)"
                        }
        with open("%s/templates/image_unit_test.gif" % os.path.dirname(os.path.realpath(__file__)), 'rb') as image_file:
            service_request['file'] = image_file
            url = "%s" % reverse('api_service_request_list')
            response = self.client.post(url,service_request)
            self.assertEqual(response.status_code, 201)

    def test_open311_serializers(self):
        """ ensure serializers can be created this way """
        ServiceRatingSerializer().data
        ServiceCommentSerializer().data
        ServiceNodeSerializer().data
        ServiceVoteSerializer().data
        ServiceListSerializer().data
        NodeRequestSerializer().data
        VoteRequestSerializer().data
        CommentRequestSerializer().data
        RatingRequestSerializer().data
        VoteRequestListSerializer().data
        CommentRequestListSerializer().data
        RatingRequestListSerializer().data
        NodeRequestListSerializer().data
        NodeRequestDetailSerializer().data

    def test_open311_UI(self):
        """ ensure open311 UI can be reached"""
        response = self.client.get(reverse('open311_demo:open311'))
        self.assertEqual(response.status_code, 200)



import simplejson as json

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.core.urlresolvers import reverse
from django.conf import settings

from nodeshot.community.notifications.models import Notification
from ..tasks import send_message


# ------ NEW NOTIFICATIONS ------ #

@receiver(post_save, sender=Notification)
def new_notification_handler(sender, **kwargs):
    if kwargs['created']:
        obj = kwargs['instance']
        message = {
            'user_id': str(obj.to_user.id),
            'model': 'notification',
            'type': obj.type,
            'url': reverse('api_notification_detail', args=[obj.id])
        }
        send_message(json.dumps(message), pipe='private')


# ------ DISCONNECT UTILITY ------ #

def disconnect():
    """ disconnect signals """
    post_save.disconnect(new_notification_handler, sender=Notification)


def reconnect():
    """ reconnect signals """
    post_save.connect(new_notification_handler, sender=Notification)


from nodeshot.core.base.settings import DISCONNECTABLE_SIGNALS
DISCONNECTABLE_SIGNALS.append(
    {
        'disconnect': disconnect,
        'reconnect': reconnect
    }
)
setattr(settings, 'NODESHOT_DISCONNECTABLE_SIGNALS', DISCONNECTABLE_SIGNALS)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Deleting field 'Layer.center'
        db.delete_column(u'layers_layer', 'center')


    def backwards(self, orm):
        # Adding field 'Layer.center'
        db.add_column(u'layers_layer', 'center',
                      self.gf('django.contrib.gis.db.models.fields.PointField')(null=True, blank=True),
                      keep_default=False)


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'layers.layer': {
            'Meta': {'object_name': 'Layer'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 10, 2, 0, 0)'}),
            'area': ('django.contrib.gis.db.models.fields.GeometryField', [], {'null': 'True'}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_external': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mantainers': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['profiles.Profile']", 'symmetrical': 'False', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'new_nodes_allowed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'nodes_minimum_distance': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'organization': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'text': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 10, 2, 0, 0)'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'profiles.profile': {
            'Meta': {'object_name': 'Profile'},
            'about': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '150', 'blank': 'True'}),
            'birth_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 10, 2, 0, 0)'}),
            'email': ('django.db.models.fields.EmailField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '1', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254', 'db_index': 'True'})
        }
    }

    complete_apps = ['layers']


from django.utils.translation import ugettext_lazy as _
from nodeshot.core.base.utils import choicify
from nodeshot.core.base.settings import ACCESS_LEVELS


__all__ = [
    'SCHEDULE_CHOICES',
    'FILTERING_CHOICES',
    'FILTER_CHOICES',
    'FILTERS',
    'OUTWARD_STATUS_CHOICES',
    'OUTWARD_STATUS',
    'GROUPS',
    'DEFAULT_GROUPS',
    'INWARD_STATUS_CHOICES'
]

SCHEDULE_CHOICES = (
    (0, _("Don't schedule, send immediately")),
    (1, _('Schedule')),
)

FILTERING_CHOICES = (
    (0, _('Send to all')),
    (1, _('Send accordingly to selected filters')),
)

FILTER_CHOICES = (
    (1, _('users of the selected groups')),
    (2, _('users which have a node in one of the selected layers')),
    (3, _('chosen users')),
)

# this is just for convenience and readability
FILTERS = {
    'groups': str(FILTER_CHOICES[0][0]),
    'layers': str(FILTER_CHOICES[1][0]),
    'users': str(FILTER_CHOICES[2][0])
}

OUTWARD_STATUS_CHOICES = (
    (-1, _('error')),
    (0, _('draft')),
    (1, _('scheduled')),
    (2, _('sent')),
    (3, _('cancelled'))
)

# this is just for convenience and readability
OUTWARD_STATUS = {
    'error': OUTWARD_STATUS_CHOICES[0][0],
    'draft': OUTWARD_STATUS_CHOICES[1][0],
    'scheduled': OUTWARD_STATUS_CHOICES[2][0],
    'sent': OUTWARD_STATUS_CHOICES[3][0],
    'cancelled': OUTWARD_STATUS_CHOICES[4][0]
}

GROUPS = []
DEFAULT_GROUPS = ''
# convert strings to integers
for group in choicify(ACCESS_LEVELS):
    GROUPS += [(int(group[0]), group[1])]
    DEFAULT_GROUPS += '%s,' % group[0]
GROUPS += [(0, _('super users'))]
DEFAULT_GROUPS += '0'

INWARD_STATUS_CHOICES = (
    (-1, _('Error')),
    (0, _('Not sent yet')),
    (1, _('Sent')),
    (2, _('Cancelled')),
)



from django.conf.urls import patterns, url
from .settings import settings


urlpatterns = patterns('nodeshot.community.mailing.views',
    # contact node
    url(r'^nodes/(?P<slug>[-\w]+)/contact/$', 'contact_node', name='api_node_contact'),
)


# contact layer
if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
    urlpatterns += patterns('nodeshot.community.mailing.views',
        url(r'^layers/(?P<slug>[-\w]+)/contact/$', 'contact_layer', name='api_layer_contact'),
    )


# contact user
if 'nodeshot.community.profiles' in settings.INSTALLED_APPS:
    urlpatterns += patterns('nodeshot.community.mailing.views',
        url(r'^profiles/(?P<username>[-.\w]+)/contact/$', 'contact_user', name='api_user_contact'),
    )



from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.core.cms.views',
    url(r'^pages/$', 'page_list', name='api_page_list'),
    url(r'^pages/(?P<slug>[-\w]+)/$', 'page_detail', name='api_page_detail'),
    url(r'^menu/$', 'menu_list', name='api_menu_list'),
)


"""
nodeshot.core.api unit tests
"""

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.core.exceptions import ImproperlyConfigured


class ParticipationModelsTest(TestCase):
    """ Models tests """
    
    fixtures = [
        'initial_data.json',
    ]
    
    def test_root_endpoint(self):
        """
        Root endpoint should be reachable
        """
        response = self.client.get(reverse('api_root_endpoint'))
        self.assertEqual(response.status_code, 200)


from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from nodeshot.core.base.models import BaseAccessLevel, BaseOrderedACL
from nodeshot.core.base.managers import AccessLevelPublishedManager


__all__ = ['Page', 'MenuItem']


ROBOTS_CHOICES = (
    ('index, follow', 'index, follow'),
    ('noindex, follow', 'noindex, follow'),
    ('index, nofollow', 'index, nofollow'),
    ('noindex, nofollow', 'noindex, nofollow'),
)


class Page(BaseAccessLevel):
    """
    Page Model
    """
    title = models.CharField(_('title'), max_length=50)
    slug = models.SlugField(_('slug'), max_length=50, blank=True, unique=True)
    content = models.TextField(_('content'))
    is_published = models.BooleanField(default=True)

    # meta fields, optional
    meta_description = models.CharField(_('meta description'), max_length=255, blank=True)
    meta_keywords = models.CharField(_('meta keywords'), max_length=255, blank=True)
    meta_robots = models.CharField(max_length=50, choices=ROBOTS_CHOICES, default=ROBOTS_CHOICES[0][0])

    objects = AccessLevelPublishedManager()

    def __unicode__(self):
        return self.title


class MenuItem(BaseOrderedACL):
    """
    MenuItem Model
    """
    name = models.CharField(_('name'), max_length=50)
    url = models.CharField(_('url'), max_length=255)
    classes = models.CharField(_('classes'), max_length=50, blank=True, help_text=_('space separated css classes'))
    parent = models.ForeignKey('self', blank=True, null=True)
    is_published = models.BooleanField(default=True)

    objects = AccessLevelPublishedManager()

    def __unicode__(self):
        return self.name

    def clean(self):
        """ support only 1 level of nesting """
        if self.parent is not None and self.parent.parent is not None:
            raise ValidationError(_('1 level of nesting only is supported'))


# ------ Signals ------ #


from django.dispatch import receiver
from django.db.models.signals import pre_delete, post_save
from nodeshot.core.base.cache import cache_delete_pattern_or_all


@receiver(post_save, sender=Page)
@receiver(pre_delete, sender=Page)
def clear_page_cache(sender, **kwargs):
    cache_delete_pattern_or_all('PageList:*')
    cache_delete_pattern_or_all('PageDetail:*')

@receiver(post_save, sender=MenuItem)
@receiver(pre_delete, sender=MenuItem)
def clear_cache_pages(sender, **kwargs):
    cache_delete_pattern_or_all('MenuList:*')



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.networking.net.models import Interface
from nodeshot.networking.net.models.choices import INTERFACE_TYPES


class Vlan(Interface):
    """ VLAN """
    tag = models.CharField(max_length=10)
    
    objects = Interface.objects.__class__()
    
    class Meta:
        app_label = 'net'
        db_table = 'net_interface_vlan'
        verbose_name = _('vlan interface')
        verbose_name_plural = _('vlan interfaces')
    
    def save(self, *args, **kwargs):
        """ automatically set Interface.type to virtual """
        self.type = INTERFACE_TYPES.get('vlan')
        super(Vlan, self).save(*args, **kwargs)


from celery import task

from django.core import management
from .settings import TEXTS


@task
def purge_notifications():
    """
    deletes old notifications
    """
    management.call_command('purge_notifications')


# ------ Asynchronous tasks ------ #


@task
def create_notifications(users, notification_model, notification_type, related_object):
    """
    create notifications in a background job to avoid slowing down users
    """
    # shortcuts for readability
    Notification = notification_model

    # text
    additional = related_object.__dict__ if related_object else ''
    notification_text = TEXTS[notification_type] % additional

    # loop users, notification settings check is done in Notification model
    for user in users:

        n = Notification(
            to_user=user,
            type=notification_type,
            text=notification_text
        )
        # attach related object if present
        if related_object:
            n.related_object = related_object
        # create notification and send according to user settings
        n.save()



from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from nodeshot.core.base.models import BaseDate
from .base import UpdateCountsMixin


class Comment(UpdateCountsMixin, BaseDate):
    """
    Comment model
    """
    node = models.ForeignKey('nodes.Node')
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    text = models.CharField(_('Comment text'), max_length=255)

    class Meta:
        app_label = 'participation'
        db_table = 'participation_comment'
        ordering = ['id']

    def __unicode__(self):
        return self.text

    def update_count(self):
        """ updates comment count """
        node_rating_count = self.node.rating_count
        node_rating_count.comment_count = self.node.comment_set.count()
        node_rating_count.save()

    def clean(self , *args, **kwargs):
        """
        Check if comments can be inserted for parent node or parent layer
        """
        # check done only for new nodes!
        if not self.pk:
            node = self.node

            # ensure comments for this node are allowed
            if  node.participation_settings.comments_allowed is False:
                raise ValidationError("Comments not allowed for this node")

            # ensure comments for this layer are allowed
            if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
                layer = node.layer
                if  layer.participation_settings.comments_allowed is False:
                    raise ValidationError("Comments not allowed for this layer")



from django.conf import settings


STATUS_MAPPING = getattr(settings, 'NODESHOT_OLDIMPORTER_STATUS_MAPPING', {
    'a': 'active',
    'h': 'active',
    'ah': 'active',
    'p': 'potential',
    'default': 'potential'
})
DEFAULT_LAYER = getattr(settings, 'NODESHOT_OLDIMPORTER_DEFAULT_LAYER', 1)



from django.conf import settings

PUBLISHED_DEFAULT = getattr(settings, 'NODESHOT_NODES_PUBLISHED_DEFAULT', True)
HSTORE_SCHEMA = getattr(settings, 'NODESHOT_NODES_HSTORE_SCHEMA', None)
REVERSION_ENABLED = getattr(settings, 'NODESHOT_NODES_REVERSION_ENABLED', True)
DESCRIPTION_HTML = getattr(settings, 'NODESHOT_NODES_HTML_DESCRIPTION', True)

if HSTORE_SCHEMA:
    ADDITIONAL_NODE_FIELDS = [field.get('name') for field in HSTORE_SCHEMA]
else:
    ADDITIONAL_NODE_FIELDS = []

ELEVATION_API_KEY = getattr(settings, 'NODESHOT_GOOGLE_ELEVATION_API_KEY', None)
ELEVATION_DEFAULT_SAMPLING = getattr(settings, 'NODESHOT_GOOGLE_ELEVATION_DEFAULT_SAMPLING', 50)



# snippet "MultiSelectField with comma separated values (Field + FormField)"
# http://djangosnippets.org/snippets/2753/

# New version of this snippet http://djangosnippets.org/snippets/1200/
# tested with Django 1.4

from django import forms
from django.db import models
from django.utils.text import capfirst
from django.core import exceptions
from django.conf import settings


class MultiSelectFormField(forms.MultipleChoiceField):
    widget = forms.CheckboxSelectMultiple
 
    def __init__(self, *args, **kwargs):
        self.max_choices = kwargs.pop('max_choices', 0)
        super(MultiSelectFormField, self).__init__(*args, **kwargs)
 
    def clean(self, value):
        if not value and self.required:
            raise forms.ValidationError(self.error_messages['required'])
        # if value and self.max_choices and len(value) > self.max_choices:
        #     raise forms.ValidationError('You must select a maximum of %s choice%s.'
        #             % (apnumber(self.max_choices), pluralize(self.max_choices)))
        return value

 
class MultiSelectField(models.Field):
    __metaclass__ = models.SubfieldBase
 
    def get_internal_type(self):
        return "CharField"
 
    def get_choices_default(self):
        return self.get_choices(include_blank=False)
 
    def _get_FIELD_display(self, field):
        value = getattr(self, field.attname)
        choicedict = dict(field.choices)
        return choicedict.get(value, value)
 
    def formfield(self, **kwargs):
        # don't call super, as that overrides default widget if it has choices
        defaults = {'required': not self.blank, 'label': capfirst(self.verbose_name),
                    'help_text': self.help_text, 'choices': self.choices}
        if self.has_default():
            defaults['initial'] = self.get_default()
        defaults.update(kwargs)
        return MultiSelectFormField(**defaults)

    def get_prep_value(self, value):
        return value

    def get_db_prep_value(self, value, connection=None, prepared=False):
        if isinstance(value, basestring):
            return value
        elif isinstance(value, list):
            return ",".join(value)
 
    def to_python(self, value):
        if value is not None:
            return value if isinstance(value, list) else value.split(',')
        return ''

    def contribute_to_class(self, cls, name):
        super(MultiSelectField, self).contribute_to_class(cls, name)
        if self.choices:
            func = lambda self, fieldname = name, choicedict = dict(self.choices): ",".join([
                choicedict.get(value, value) for value in getattr(self, fieldname)
            ])
            setattr(cls, 'get_%s_display' % self.name, func)
 
    def validate(self, value, model_instance):
        arr_choices = self.get_choices_selected(self.get_choices_default())
        try:
            for opt_select in value:
                if (int(opt_select) not in arr_choices):  # the int() here is for comparing with integer choices
                    raise exceptions.ValidationError(self.error_messages['invalid_choice'] % value)
        except ValueError:
            if not self.blank:
                raise exceptions.ValidationError(self.error_messages['invalid_choice'] % value)
        return
 
    def get_choices_selected(self, arr_choices=''):
        if not arr_choices:
            return False
        list = []
        for choice_selected in arr_choices:
            list.append(choice_selected[0])
        return list
 
    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)


# taken from django-colorful
# https://github.com/charettes/django-colorful
import re
from django.forms.fields import RegexField
from .widgets import ColorFieldWidget

RGB_REGEX = re.compile('^#?((?:[0-F]{3}){1,2})$', re.IGNORECASE)

class RGBColorField(models.CharField):

    widget = ColorFieldWidget

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 7
        super(RGBColorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.update({
                       'form_class': RegexField,
                       'widget': self.widget,
                       'regex': RGB_REGEX
                       })
        return super(RGBColorField, self).formfield(**kwargs)


# needed for South compatibility
if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^coop\.utils\.fields\.MultiSelectField"])
    add_introspection_rules([], ["^nodeshot\.core\.base\.fields\.RGBColorField"])


from django.conf import settings

ACCESS_LEVELS = getattr(settings, 'NODESHOT_ACCESS_LEVELS', {
    'registered': 1,
    'community': 2,
    'trusted': 3
})
ACL_DEFAULT_VALUE = getattr(settings, 'NODESHOT_ACL_DEFAULT_VALUE', 'public')
ACL_DEFAULT_EDITABLE  = getattr(settings, 'NODESHOT_ACL_DEFAULT_EDITABLE', True)
DISCONNECTABLE_SIGNALS = getattr(settings, 'NODESHOT_DISCONNECTABLE_SIGNALS', [])



from django.conf import settings

INFLUXDB_HOST = getattr(settings, 'INFLUXDB_HOST', 'localhost')
INFLUXDB_PORT = getattr(settings, 'INFLUXDB_PORT', '8086')
INFLUXDB_USER = getattr(settings, 'INFLUXDB_USER')
INFLUXDB_PASSWORD = getattr(settings, 'INFLUXDB_PASSWORD')
INFLUXDB_DATABASE = getattr(settings, 'INFLUXDB_DATABASE')
INFLUXDB_FAIL_SILENTLY = getattr(settings, 'INFLUXDB_FAIL_SILENTLY', False)
INFLUXDB_MIDDLEWARE_IGNORED_MODULES = getattr(settings, 'INFLUXDB_MIDDLEWARE_IGNORED_MODULES', [
    'django.contrib.staticfiles.views'
])



from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from nodeshot.networking.net.models import Interface
from nodeshot.networking.net.models.choices import INTERFACE_TYPES


class Bridge(Interface):
    """ Bridge interface """
    interfaces = models.ManyToManyField(Interface, verbose_name=_('interfaces'), related_name='bridge_interfaces')
    
    objects = Interface.objects.__class__()
    
    class Meta:
        app_label = 'net'
        db_table = 'net_interface_bridge'
        verbose_name = _('bridge interface')
        verbose_name_plural = _('bridge interfaces')
    
    def save(self, *args, **kwargs):
        """ automatically set Interface.type to bridge """
        self.type = INTERFACE_TYPES.get('bridge')
        super(Bridge, self).save(*args, **kwargs)


from django.dispatch import receiver
from django.db.models.signals import m2m_changed

@receiver(m2m_changed, sender=Bridge.interfaces.through)
def validate_bridged_interfaces(sender, instance, action, reverse, model, pk_set, **kwargs):
    """
    Bridge interface validation constraints:
        * can't bridge interfaces of different devices
        * can't bridge self
        * there must be at least two interfaces bridged
    """
    if action != 'pre_add':
        return
    
    interfaces = Interface.objects.filter(pk__in=pk_set)
    device = instance.device
    device_id = instance.device.id
    invalid_interfaces = []
    trying_to_bridge_self = False
    
    for interface in interfaces:
        # interfaces must belong to the same device
        if interface.device_id != device_id:
            invalid_interfaces.append(interface.mac)
        # you can't bridge same interface
        elif interface.id == instance.id:
            trying_to_bridge_self = True
    
    # ensure at least two interfaces are present
    if instance.interfaces.count() < 2 and len(pk_set) < 2:
        raise ValidationError(_(u'You must bridge at least 2 interfaces'))
    
    if invalid_interfaces:
        raise ValidationError(_(u'The interface%s %s %s not belong to device "%s"' % (
            "s" if len(invalid_interfaces) > 1 else "",
            ", ".join(invalid_interfaces),
            "do" if len(invalid_interfaces) > 1 else "does",
            device.name
        )))
    elif trying_to_bridge_self:
        raise ValidationError(_(u'Cannot bridge interface %s because that is the actual interface you are editing''' % instance.mac))


from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.networking.net.models import Interface
from nodeshot.networking.net.models.choices import ETHERNET_STANDARDS, DUPLEX_CHOICES, INTERFACE_TYPES


class Ethernet(Interface):
    """ Ethernet Interface """
    standard = models.CharField(_('standard'), max_length=15, choices=ETHERNET_STANDARDS)
    duplex = models.CharField(_('duplex?'), max_length=15, choices=DUPLEX_CHOICES)
    
    objects = Interface.objects.__class__()
    
    class Meta:
        app_label = 'net'
        db_table = 'net_interface_ethernet'
        verbose_name = _('ethernet interface')
        verbose_name_plural = _('ethernet interfaces')
    
    def save(self, *args, **kwargs):
        """ automatically set Interface.type to ethernet """
        self.type = INTERFACE_TYPES.get('ethernet')
        super(Ethernet, self).save(*args, **kwargs)


from django.contrib.admin.sites import site
from nodeshot.core.layers.models import Layer

from .models import LayerExternal
from .admin import LayerAdmin


def layer_external_reload_schema(request, pk):
    # if POST and synchronizer_path param
    if request.method == 'POST' and request.POST.get('synchronizer_path'):
        layer = Layer.objects.get(pk=pk)
        # get external layer or init a new one
        try:
            external = layer.external
        except LayerExternal.DoesNotExist:
            external = LayerExternal(layer=layer)
        # reset schema
        external.synchronizer_path = 'None'
        external._reload_schema()
        # set synchronizer_path
        external.synchronizer_path = request.POST.get('synchronizer_path')
        external.save()
        # pass a GET request to the admin view
        request.method = 'GET'
    # return layer admin change view instance
    return LayerAdmin(Layer, site).change_view(request, pk)



from django.test.client import FakePayload, MULTIPART_CONTENT
from django.test.client import Client as BaseClient
from django.test import TestCase
from django.conf import settings

from urlparse import urlparse, urlsplit


if 'nodeshot.community.profiles' in settings.INSTALLED_APPS:
    user_fixtures = 'test_profiles.json'
else:
    user_fixtures = 'test_users.json'


### --- Add patch method, for Django < 1.5 --- ###


class Client(BaseClient):
    """
    Construct a second test client which can do PATCH requests.
    """
    def patch(self, path, data={}, content_type=MULTIPART_CONTENT, **extra):
        "Construct a PATCH request."
 
        patch_data = self._encode_data(data, content_type)
 
        parsed = urlparse(path)
        r = {
            'CONTENT_LENGTH': len(patch_data),
            'CONTENT_TYPE':   content_type,
            'PATH_INFO':      self._get_path(parsed),
            'QUERY_STRING':   parsed[4],
            'REQUEST_METHOD': 'PATCH',
            'wsgi.input':     FakePayload(patch_data),
        }
        r.update(extra)
        return self.request(**r)


class BaseTestCase(TestCase):
    """
    Test case with a client that can do patch requests
    """
    
    client_class = Client


from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError, ImproperlyConfigured, ObjectDoesNotExist
from django.utils.module_loading import import_by_path

from django_hstore.fields import DictionaryField

from ..settings import SYNCHRONIZERS


# choices
SYNCHRONIZERS = [
    ('None', _('None'))
] + SYNCHRONIZERS


class LayerExternal(models.Model):
    """
    External Layers, extend 'Layers' with additional files
    These are the layers that are managed by local groups or other organizations
    """
    layer = models.OneToOneField('layers.Layer', verbose_name=_('layer'), parent_link=True, related_name='external')
    synchronizer_path = models.CharField(_('synchronizer'), max_length=128, choices=SYNCHRONIZERS, default='None')
    config = DictionaryField(_('configuration'),
                             help_text=_('Synchronizer specific configuration (eg: API URL, auth info, ecc)'),
                             blank=True,
                             null=True,
                             editable=False,
                             schema=None)
    # private attributes that will hold synchronizer info
    _synchronizer = None
    _synchronizer_class = None

    class Meta:
        app_label = 'sync'
        db_table = 'layers_external'
        verbose_name = _('external layer')
        verbose_name_plural = _('external layer info')

    def __unicode__(self):
        try:
            return '%s external layer config' % self.layer.name
        except ObjectDoesNotExist:
            return 'LayerExternal object'

    def __init__(self, *args, **kwargs):
        """ custom init method """
        super(LayerExternal, self).__init__(*args, **kwargs)
        # avoid blocking page loading in case of missing required config keys in configuration
        try:
            synchronizer = self.synchronizer
        except ImproperlyConfigured:
            synchronizer = False
        # if synchronizer has get_nodes method
        # add get_nodes method to current LayerExternal instance
        if synchronizer is not False and hasattr(synchronizer, 'get_nodes'):
            self.get_nodes = synchronizer.get_nodes
        # load schema
        self._reload_schema()

    def _reload_schema(self):
        if self.synchronizer_class:
            schema = self.synchronizer_class.SCHEMA
        else:
            schema = None
        if self.config.field.schema is not schema:
            self.config.field.reload_schema(schema)
            # if schema is None set editable to False
            if schema is None:
                self.config.field.editable = False

    def clean(self, *args, **kwargs):
        """
        Call self.synchronizer.clean method
        """
        if self.synchronizer_path != 'None' and self.config:
            # call synchronizer custom clean
            try:
                self.synchronizer.load_config(self.config)
                self.synchronizer.clean()
            except ImproperlyConfigured as e:
                raise ValidationError(e.message)

    def save(self, *args, **kwargs):
        """
        call synchronizer "after_external_layer_saved" method
        for any additional operation that must be executed after save
        """
        after_save = kwargs.pop('after_save', True)
        super(LayerExternal, self).save(*args, **kwargs)
        # call after_external_layer_saved method of synchronizer
        if after_save:
            try:
                synchronizer = self.synchronizer
            except ImproperlyConfigured:
                pass
            else:
                if synchronizer:
                    synchronizer.after_external_layer_saved(self.config)
        # reload schema
        self._reload_schema()

    @property
    def synchronizer(self):
        """ access synchronizer """
        if not self.synchronizer_path or self.synchronizer_path == 'None' or not self.layer:
            return False
        # ensure data is up to date
        if (self._synchronizer is not None and self._synchronizer_class.__name__ not in self.synchronizer_path):
            self._synchronizer = None
            self._synchronizer_class = None
        # init synchronizer only if necessary
        if not self._synchronizer:
            self._synchronizer = (self.synchronizer_class)(self.layer)
        return self._synchronizer

    @property
    def synchronizer_class(self):
        """ returns synchronizer class """
        if not self.synchronizer_path or self.synchronizer_path == 'None' or not self.layer:
            return False
        # ensure data is up to date
        if (self._synchronizer_class is not None and self._synchronizer_class.__name__ not in self.synchronizer_path):
            self._synchronizer = None
            self._synchronizer_class = None
        # import synchronizer class only if not imported already
        if not self._synchronizer_class:
            self._synchronizer_class = import_by_path(self.synchronizer_path)
        return self._synchronizer_class



from netfields import MACAddressField

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from nodeshot.core.base.models import BaseAccessLevel
from choices import INTERFACE_TYPE_CHOICES

from django_hstore.fields import DictionaryField, ReferencesField
from nodeshot.core.base.managers import HStoreAccessLevelManager as InterfaceManager


class Interface(BaseAccessLevel):
    """ Interface model """
    device = models.ForeignKey('net.Device')
    type = models.IntegerField(_('type'), max_length=2, choices=INTERFACE_TYPE_CHOICES, blank=True)
    name = models.CharField(_('name'), max_length=10, blank=True, null=True)
    mac = MACAddressField(_('mac address'), max_length=17, unique=True, default=None, null=True, blank=True)
    mtu = models.IntegerField(_('MTU'), blank=True, null=True, default=1500,
                              help_text=_('Maximum Trasmission Unit'))
    tx_rate = models.IntegerField(_('TX Rate'), null=True, default=None, blank=True)
    rx_rate = models.IntegerField(_('RX Rate'), null=True, default=None, blank=True)
    
    # extra data
    data = DictionaryField(_('extra data'), null=True, blank=True,
                           help_text=_('store extra attributes in JSON string'))
    shortcuts = ReferencesField(null=True, blank=True)
    
    objects = InterfaceManager()
    
    class Meta:
        app_label = 'net'
    
    def __unicode__(self):
        return '%s %s' % (self.get_type_display(), self.mac)
    
    def save(self, *args, **kwargs):
        """
        Custom save method does the following:
            * save shortcuts if HSTORE is enabled
        """
        if 'node' not in self.shortcuts:
            self.shortcuts['node'] = self.device.node
        
        if 'user' not in self.shortcuts:
            self.shortcuts['user'] = self.device.node.user
        
        if 'layer' not in self.shortcuts and 'nodeshot.core.layers' in settings.INSTALLED_APPS:
            self.shortcuts['layer'] = self.device.node.layer
        
        super(Interface, self).save(*args, **kwargs)
    
    @property
    def owner(self):
        if 'user' not in self.shortcuts:
            if self.device or self.device_id:
                self.save()
            else:
                raise Exception('Instance does not have a device set yet')
        
        return self.shortcuts['user']
    
    @property
    def node(self):
        if 'node' not in self.shortcuts:
            if self.device or self.device_id:
                self.save()
            else:
                raise Exception('Instance does not have a device set yet')
        return self.shortcuts['node']
    
    @property
    def layer(self):
        if 'nodeshot.core.layers' not in settings.INSTALLED_APPS:
            return False
        if 'layer' not in self.shortcuts:
            if self.device or self.device_id:
                self.save()
            else:
                raise Exception('Instance does not have a device set yet')
        return self.shortcuts['layer']
    
    @property
    def ip_addresses(self):
        try:
            addresses = self.data.get('ip_addresses', '')
        # self.data might be none, hence self.data['ip_addresses'] will raise an exception
        except AttributeError:
            addresses = ''
        return addresses.replace(' ', '').split(',') if addresses else []
    
    @ip_addresses.setter
    def ip_addresses(self, value):
        """ :param value: a list of ip addresses """
        if not isinstance(value, list):
            raise ValueError('ip_addresses value must be a list')
        # in soem cases self.data might be none, so let's instantiate an empty dict
        if self.data is None:
            self.data = {}
        # update field
        self.data['ip_addresses'] = ', '.join(value)
    
    if 'grappelli' in settings.INSTALLED_APPS:
        @staticmethod
        def autocomplete_search_fields():
            return ('mac__icontains', 'data__icontains')



"""
OpenWISP OAuth support.
"""
from urllib import urlencode
import simplejson as json

from django.core.exceptions import ImproperlyConfigured

from social_auth.utils import dsa_urlopen
from social_auth.backends import BaseOAuth2, OAuthBackend

from ..settings import settings

# OpenWISP configuration
try:
    OPENWISP_BASE_URL = settings.OPENWISP_BASE_URL
except AttributeError:
    raise ImproperlyConfigured('missing settings.OPENWISP_BASE_URL')

OPENWISP_AUTHORIZATION_URL = '%s/oauth/authorize' % OPENWISP_BASE_URL
OPENWISP_ACCESS_TOKEN_URL = '%s/oauth/access_token' % OPENWISP_BASE_URL
OPENWISP_USER_DATA_URL = '%s/oauth/account_details.json' % OPENWISP_BASE_URL


class OpenWISPBackend(OAuthBackend):
    """OpenWISP OAuth authentication backend"""
    name = 'openwisp'
    # Default extra data to store
    EXTRA_DATA = [
        ('id', 'id'),
        ('expires', 'expires')
    ]

    def get_user_details(self, response):
        """Return user details from OpenWISP account"""
        return {
            'username': response.get('username'),
            'email': response.get('email'),
            'first_name': response.get('first_name'),
            'last_name': response.get('last_name'),
            'birth_date': response.get('birth_date'),
        }


class OpenWISPAuth(BaseOAuth2):
    """OpenWISP OAuth2 mechanism"""
    AUTHORIZATION_URL = OPENWISP_AUTHORIZATION_URL
    ACCESS_TOKEN_URL = OPENWISP_ACCESS_TOKEN_URL
    AUTH_BACKEND = OpenWISPBackend
    SETTINGS_KEY_NAME = 'OPENWISP_APP_ID'
    SETTINGS_SECRET_NAME = 'OPENWISP_API_SECRET'
    SCOPE_SEPARATOR = ','
    SCOPE_VAR_NAME = 'OPENWISP_EXTENDED_PERMISSIONS'

    def user_data(self, access_token, *args, **kwargs):
        """Loads user data from service"""
        url = OPENWISP_USER_DATA_URL + '?' + urlencode({
            'access_token': access_token
        })

        try:
            data = json.load(dsa_urlopen(url))
        except ValueError:
            data = None

        return data

# Backend definition
BACKENDS = {
    'openwisp': OpenWISPAuth,
}



from rest_framework import permissions


class IsOwnerOrReadOnly(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has an `owner` attribute.
    """

    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True
        
        action = ''
        
        if request.method in ['PUT', 'PATCH']:
            action = 'change'
        elif request.method in ['DELETE']:
            action = 'delete'
        elif request.method == 'POST':
            action = 'add'
        
        class_name = obj.__class__.__name__
        
        # Instance must have an attribute named `owner`.
        return obj.owner == request.user or request.user.has_perm('net.%s_%s' % (action, class_name.lower()))


from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.networking.net.models import Interface
from nodeshot.networking.net.models.choices import WIRELESS_MODE, WIRELESS_STANDARDS, WIRELESS_CHANNEL, INTERFACE_TYPES


class Wireless(Interface):
    """ Wireless Interface """
    mode = models.CharField(_('wireless mode'), max_length=5,
                            choices=WIRELESS_MODE,
                            blank=True, null=True, default=None)
    standard = models.CharField(_('wireless standard'), max_length=7,
                                choices=WIRELESS_STANDARDS,
                                blank=True, null=True, default=None)
    channel = models.CharField(_('channel'), max_length=4,
                               choices=WIRELESS_CHANNEL,
                               blank=True, null=True, default=None)
    channel_width = models.CharField(_('channel width'), max_length=6,
                                     blank=True, null=True)
    output_power = models.IntegerField(_('output power'), null=True, blank=True)
    dbm = models.IntegerField(_('dBm'), null=True, default=None, blank=True)
    noise = models.IntegerField(_('noise'), null=True, default=None, blank=True)
    
    objects = Interface.objects.__class__()
    
    class Meta:
        app_label = 'net'
        db_table = 'net_interface_wireless'
        verbose_name = _('wireless interface')
        verbose_name_plural = _('wireless interfaces')
    
    def save(self, *args, **kwargs):
        """ automatically set Interface.type to wireless """
        self.type = INTERFACE_TYPES.get('wireless')
        super(Wireless, self).save(*args, **kwargs)


# This application object is used by any WSGI server configured to use this
# file. This includes Django's development server, if the WSGI_APPLICATION
# setting points here.

from django.conf import settings


# sentry WSGI handler if configured
# more info here: http://raven.readthedocs.org/en/latest/config/django.html#wsgi-middleware
if getattr(settings, 'RAVEN_CONFIG', {}):
    from raven.contrib.django.raven_compat.middleware.wsgi import Sentry
    from django.core.handlers.wsgi import WSGIHandler
    application = Sentry(WSGIHandler())
# django default wsgi
else:
    from django.core.wsgi import get_wsgi_application
    application = get_wsgi_application()
